@@ -10,8 +10,14 @@ import os
 import sys
 import argparse
 
+WARNING = "\033[93m"
+ENDC = "\033[0m"
+
 # Minimum version of markdown required
-MINIMUM_MARKDOWN = (3, 3, 3)
+MINIMUM_MARKDOWN = (3, 2, 0)
+
+# Minimum meson required
+MINIMUM_MESON = "0.61.0"
 
 
 def get_possible_profiles():
@@ -30,7 +36,16 @@ def detect_profile():
     return target
 
 
-def test_markdown():
+def pip_install_package(debug, name):
+    import subprocess
+
+    cmd = ["python3", "-m", "pip", "install", "--upgrade", name]
+    if debug:
+        print(cmd)
+    subprocess.call(cmd)
+
+
+def test_markdown(debug):
     try:
         import markdown
 
@@ -39,10 +54,30 @@ def test_markdown():
         new_enough = False
     if not new_enough:
         print("python3-markdown must be installed/upgraded")
-    sys.exit(not new_enough)
+        pip_install_package(debug, "markdown")
 
 
-def parse_dependencies(OS, SUBOS, requested_type):
+def test_meson(debug):
+    from importlib.metadata import version, PackageNotFoundError
+
+    try:
+        new_enough = version("meson") >= MINIMUM_MESON
+    except PackageNotFoundError:
+        import subprocess
+
+        try:
+            ver = (
+                subprocess.check_output(["meson", "--version"]).strip().decode("utf-8")
+            )
+            new_enough = ver >= MINIMUM_MESON
+        except FileNotFoundError:
+            new_enough = False
+    if not new_enough:
+        print("meson must be installed/upgraded")
+        pip_install_package(debug, "meson")
+
+
+def parse_dependencies(OS, variant, add_control):
     import xml.etree.ElementTree as etree
 
     deps = []
@@ -51,31 +86,69 @@ def parse_dependencies(OS, SUBOS, requested_type):
     tree = etree.parse(os.path.join(directory, "dependencies.xml"))
     root = tree.getroot()
     for child in root:
-        if "type" not in child.attrib or "id" not in child.attrib:
+        if "id" not in child.attrib:
             continue
         for distro in child:
             if "id" not in distro.attrib:
                 continue
             if distro.attrib["id"] != OS:
                 continue
-            packages = distro.findall("package")
-            for package in packages:
-                if SUBOS:
+            control = ""
+            if add_control:
+                inclusive = []
+                exclusive = []
+                if not distro.findall("control"):
+                    continue
+                for control_parent in distro.findall("control"):
+                    for obj in control_parent.findall("inclusive"):
+                        inclusive.append(obj.text)
+                    for obj in control_parent.findall("exclusive"):
+                        exclusive.append(obj.text)
+                if inclusive or exclusive:
+                    inclusive = " ".join(inclusive).strip()
+                    exclusive = " !".join(exclusive).strip()
+                    if exclusive:
+                        exclusive = "!%s" % exclusive
+                    control = " [%s%s]" % (inclusive, exclusive)
+            for package in distro.findall("package"):
+                if variant:
                     if "variant" not in package.attrib:
                         continue
-                    if package.attrib["variant"] != SUBOS:
+                    if package.attrib["variant"] != variant:
                         continue
                 if package.text:
                     dep = package.text
                 else:
                     dep = child.attrib["id"]
-                if child.attrib["type"] == requested_type and dep:
+                dep += control
+                if dep:
                     deps.append(dep)
     return deps
 
 
+def _validate_deps(os, deps):
+    validated = deps
+    if os == "debian" or os == "ubuntu":
+        try:
+            from apt import cache
+
+            cache = cache.Cache()
+            for pkg in deps:
+                if not cache.has_key(pkg) and not cache.is_virtual_package(pkg):
+                    print(
+                        f"{WARNING}WARNING:{ENDC} ignoring unavailable package %s" % pkg
+                    )
+                    validated.remove(pkg)
+        except ModuleNotFoundError:
+            print(
+                f"{WARNING}WARNING:{ENDC} Unable to validate package dependency list without python3-apt"
+            )
+    return validated
+
+
 def get_build_dependencies(os, variant):
-    return parse_dependencies(os, variant, "build")
+    parsed = parse_dependencies(os, variant, False)
+    return _validate_deps(os, parsed)
 
 
 def _get_installer_cmd(os, yes):
@@ -122,6 +195,7 @@ if __name__ == "__main__":
             choices=[
                 "get-dependencies",
                 "test-markdown",
+                "test-meson",
                 "detect-profile",
                 "install-dependencies",
                 "install-pip",
@@ -146,15 +220,17 @@ if __name__ == "__main__":
     )
     args = parser.parse_args()
 
-    # these require variants to be set
-    if not args.variant and (args.os == "ubuntu" or args.os == "debian"):
+    # fall back in all cases
+    if not args.variant:
         args.variant = os.uname().machine
     if not command:
         command = args.command
 
     # command to run
     if command == "test-markdown":
-        test_markdown()
+        test_markdown(args.debug)
+    elif command == "test-meson":
+        test_meson(args.debug)
     elif command == "detect-profile":
         print(detect_profile())
     elif command == "get-dependencies":
