@@ -140,44 +140,58 @@ fwupd_checksum_get_best(GPtrArray *checksums)
 	return NULL;
 }
 
+static gchar *
+fwupd_get_os_release_filename(void)
+{
+#ifndef _WIN32
+	const gchar *sysconfdir = g_getenv("FWUPD_SYSCONFDIR");
+	g_autofree gchar *fn1 = NULL;
+
+	/* override */
+	if (sysconfdir != NULL) {
+		g_autofree gchar *fn2 = g_build_filename(sysconfdir, "os-release", NULL);
+		if (g_file_test(fn2, G_FILE_TEST_EXISTS))
+			return g_steal_pointer(&fn2);
+	}
+
+	/* host locations */
+	if (g_strcmp0(sysconfdir, "/etc") != 0) {
+		g_autofree gchar *fn2 = g_strdup("/etc/os-release");
+		if (g_file_test(fn2, G_FILE_TEST_EXISTS))
+			return g_steal_pointer(&fn2);
+	}
+	fn1 = g_strdup("/usr/lib/os-release");
+	if (g_file_test(fn1, G_FILE_TEST_EXISTS))
+		return g_steal_pointer(&fn1);
+#endif
+	return NULL;
+}
+
 /**
- * fwupd_get_os_release:
+ * fwupd_get_os_release_full:
+ * @filename: (nullable): optional filename to load
  * @error: (nullable): optional return location for an error
  *
- * Loads information from the system os-release file.
+ * Loads information from a defined system os-release file.
  *
  * Returns: (transfer container) (element-type utf8 utf8): keys from os-release
  *
- * Since: 1.0.7
+ * Since: 1.8.8
  **/
 GHashTable *
-fwupd_get_os_release(GError **error)
+fwupd_get_os_release_full(const gchar *filename, GError **error)
 {
-	const gchar *filename = NULL;
-	const gchar *sysconfdir = g_getenv("FWUPD_SYSCONFDIR");
 	g_autofree gchar *buf = NULL;
+	g_autofree gchar *filename2 = g_strdup(filename);
 	g_auto(GStrv) lines = NULL;
 	g_autoptr(GHashTable) hash = NULL;
-	g_autoptr(GPtrArray) paths = g_ptr_array_new_with_free_func(g_free);
 
 	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
-	/* find the correct file */
-	if (sysconfdir != NULL)
-		g_ptr_array_add(paths, g_build_filename(sysconfdir, "os-release", NULL));
-	if (g_strcmp0(sysconfdir, "/etc") != 0)
-		g_ptr_array_add(paths, g_strdup("/etc/os-release"));
-	g_ptr_array_add(paths, g_strdup("/usr/lib/os-release"));
-	for (guint i = 0; i < paths->len; i++) {
-		const gchar *path = g_ptr_array_index(paths, i);
-		if (g_file_test(path, G_FILE_TEST_EXISTS)) {
-			filename = path;
-			break;
-		}
-	}
-
 	hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-	if (filename == NULL) {
+	if (filename2 == NULL)
+		filename2 = fwupd_get_os_release_filename();
+	if (filename2 == NULL) {
 #if defined(_WIN32)
 		/* TODO: Read the Windows version */
 		g_hash_table_insert(hash, g_strdup("OS"), g_strdup("Windows"));
@@ -193,7 +207,7 @@ fwupd_get_os_release(GError **error)
 	}
 
 	/* load each line */
-	if (!g_file_get_contents(filename, &buf, NULL, error))
+	if (!g_file_get_contents(filename2, &buf, NULL, error))
 		return NULL;
 	lines = g_strsplit(buf, "\n", -1);
 	for (guint i = 0; lines[i] != NULL; i++) {
@@ -218,6 +232,23 @@ fwupd_get_os_release(GError **error)
 	return g_steal_pointer(&hash);
 }
 
+/**
+ * fwupd_get_os_release:
+ * @error: (nullable): optional return location for an error
+ *
+ * Loads information from the system os-release file.
+ *
+ * Returns: (transfer container) (element-type utf8 utf8): keys from os-release
+ *
+ * Since: 1.0.7
+ **/
+GHashTable *
+fwupd_get_os_release(GError **error)
+{
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+	return fwupd_get_os_release_full(NULL, error);
+}
+
 static gchar *
 fwupd_build_user_agent_os_release(void)
 {
@@ -417,14 +448,20 @@ static void
 fwupd_build_history_report_json_device(JsonBuilder *builder, FwupdDevice *dev)
 {
 	FwupdRelease *rel = fwupd_device_get_release_default(dev);
+	GChecksumType checksum_types[] = {G_CHECKSUM_SHA256, G_CHECKSUM_SHA1, 0};
 	GPtrArray *checksums;
 	GPtrArray *guids;
 
 	/* identify the firmware used */
-	json_builder_set_member_name(builder, "Checksum");
 	checksums = fwupd_release_get_checksums(rel);
-	json_builder_add_string_value(builder,
-				      fwupd_checksum_get_by_kind(checksums, G_CHECKSUM_SHA1));
+	for (guint i = 0; checksum_types[i] != 0; i++) {
+		const gchar *checksum = fwupd_checksum_get_by_kind(checksums, checksum_types[i]);
+		if (checksum != NULL) {
+			json_builder_set_member_name(builder, "Checksum");
+			json_builder_add_string_value(builder, checksum);
+			break;
+		}
+	}
 
 	/* identify the firmware written */
 	checksums = fwupd_device_get_checksums(dev);
@@ -1202,3 +1239,51 @@ fwupd_common_json_add_stringv(JsonBuilder *builder, const gchar *key, gchar **va
 		json_builder_add_string_value(builder, value[i]);
 	json_builder_end_array(builder);
 }
+
+/**
+ * fwupd_pad_kv_str: (skip):
+ **/
+void
+fwupd_pad_kv_str(GString *str, const gchar *key, const gchar *value)
+{
+	/* ignore */
+	if (key == NULL || value == NULL)
+		return;
+	g_string_append_printf(str, "  %s: ", key);
+	for (gsize i = strlen(key); i < 20; i++)
+		g_string_append(str, " ");
+	g_string_append_printf(str, "%s\n", value);
+}
+
+/**
+ * fwupd_pad_kv_unx: (skip):
+ **/
+void
+fwupd_pad_kv_unx(GString *str, const gchar *key, guint64 value)
+{
+	g_autoptr(GDateTime) date = NULL;
+	g_autofree gchar *tmp = NULL;
+
+	/* ignore */
+	if (value == 0)
+		return;
+
+	date = g_date_time_new_from_unix_utc((gint64)value);
+	tmp = g_date_time_format(date, "%F");
+	fwupd_pad_kv_str(str, key, tmp);
+}
+
+/**
+ * fwupd_pad_kv_int: (skip):
+ **/
+void
+fwupd_pad_kv_int(GString *str, const gchar *key, guint32 value)
+{
+	g_autofree gchar *tmp = NULL;
+
+	/* ignore */
+	if (value == 0)
+		return;
+	tmp = g_strdup_printf("%" G_GUINT32_FORMAT, value);
+	fwupd_pad_kv_str(str, key, tmp);
+}
