@@ -22,6 +22,7 @@
 #include "fwupd-plugin-private.h"
 #include "fwupd-release-private.h"
 #include "fwupd-remote-private.h"
+#include "fwupd-report-private.h"
 #include "fwupd-request-private.h"
 #include "fwupd-security-attr-private.h"
 
@@ -170,7 +171,7 @@ fwupd_enums_func(void)
 		g_assert_cmpstr(tmp, !=, NULL);
 		g_assert_cmpint(fwupd_device_problem_from_string(tmp), ==, i);
 	}
-	for (guint64 i = 1; i <= FWUPD_PLUGIN_FLAG_AUTH_REQUIRED; i *= 2) {
+	for (guint64 i = 1; i <= FWUPD_PLUGIN_FLAG_MEASURE_SYSTEM_INTEGRITY; i *= 2) {
 		const gchar *tmp = fwupd_plugin_flag_to_string(i);
 		if (tmp == NULL)
 			g_warning("missing plugin flag 0x%x", (guint)i);
@@ -198,6 +199,13 @@ fwupd_enums_func(void)
 		g_assert_cmpstr(tmp, !=, NULL);
 		g_assert_cmpint(fwupd_release_flag_from_string(tmp), ==, i);
 	}
+	for (guint64 i = 1; i <= FWUPD_REQUEST_FLAG_ALLOW_GENERIC_IMAGE; i *= 2) {
+		const gchar *tmp = fwupd_request_flag_to_string(i);
+		if (tmp == NULL)
+			g_warning("missing request flag 0x%x", (guint)i);
+		g_assert_cmpstr(tmp, !=, NULL);
+		g_assert_cmpint(fwupd_request_flag_from_string(tmp), ==, i);
+	}
 	for (guint i = 1; i < FWUPD_KEYRING_KIND_LAST; i++) {
 		const gchar *tmp = fwupd_keyring_kind_to_string(i);
 		if (tmp == NULL)
@@ -582,6 +590,7 @@ fwupd_release_func(void)
 	g_autofree gchar *str = NULL;
 	g_autoptr(FwupdRelease) release1 = NULL;
 	g_autoptr(FwupdRelease) release2 = NULL;
+	g_autoptr(FwupdRelease) release3 = fwupd_release_new();
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GVariant) data = NULL;
 
@@ -712,6 +721,99 @@ fwupd_release_func(void)
 				    &error);
 	g_assert_no_error(error);
 	g_assert_true(ret);
+
+	/* copy properties */
+	fwupd_release_incorporate(release3, release2);
+	g_assert_cmpstr(fwupd_release_get_metadata_item(release3, "foo"), ==, "bar");
+	g_assert_cmpstr(fwupd_release_get_remote_id(release3), ==, "remote-id");
+	g_assert_cmpstr(fwupd_release_get_appstream_id(release3), ==, "appstream-id");
+	g_assert_cmpstr(fwupd_release_get_id(release3), ==, "id");
+}
+
+static gchar *
+fwupd_report_to_json_string(FwupdReport *report, GError **error)
+{
+	g_autofree gchar *data = NULL;
+	g_autoptr(JsonGenerator) json_generator = NULL;
+	g_autoptr(JsonBuilder) builder = json_builder_new();
+	g_autoptr(JsonNode) json_root = NULL;
+	json_builder_begin_object(builder);
+	fwupd_report_to_json(report, builder);
+	json_builder_end_object(builder);
+	json_root = json_builder_get_root(builder);
+	json_generator = json_generator_new();
+	json_generator_set_pretty(json_generator, TRUE);
+	json_generator_set_root(json_generator, json_root);
+	data = json_generator_to_data(json_generator, NULL);
+	if (data == NULL) {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INTERNAL,
+			    "Failed to convert report to json.");
+		return NULL;
+	}
+	return g_steal_pointer(&data);
+}
+
+static void
+fwupd_report_func(void)
+{
+	gboolean ret;
+	g_autofree gchar *json1 = NULL;
+	g_autofree gchar *json2 = NULL;
+	g_autofree gchar *str = NULL;
+	g_autoptr(FwupdReport) report1 = NULL;
+	g_autoptr(FwupdReport) report2 = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GVariant) data = NULL;
+
+	report1 = fwupd_report_new();
+	fwupd_report_add_metadata_item(report1, "foo", "bar");
+	fwupd_report_add_metadata_item(report1, "baz", "bam");
+	fwupd_report_set_version_old(report1, "1.2.3");
+	fwupd_report_set_created(report1, 5678);
+	fwupd_report_set_vendor(report1, "acme");
+	fwupd_report_set_vendor_id(report1, 2468);
+	fwupd_report_set_device_name(report1, "name");
+	fwupd_report_set_distro_id(report1, "distro_id");
+	fwupd_report_set_distro_version(report1, "distro_version");
+	data = fwupd_report_to_variant(report1);
+	report2 = fwupd_report_from_variant(data);
+	g_assert_cmpstr(fwupd_report_get_metadata_item(report2, "foo"), ==, "bar");
+	g_assert_cmpstr(fwupd_report_get_metadata_item(report2, "baz"), ==, "bam");
+	g_assert_cmpstr(fwupd_report_get_version_old(report2), ==, "1.2.3");
+	g_assert_cmpstr(fwupd_report_get_vendor(report2), ==, "acme");
+	g_assert_cmpint(fwupd_report_get_vendor_id(report2), ==, 2468);
+	g_assert_cmpstr(fwupd_report_get_device_name(report2), ==, "name");
+	g_assert_cmpstr(fwupd_report_get_distro_id(report2), ==, "distro_id");
+	g_assert_cmpstr(fwupd_report_get_distro_version(report2), ==, "distro_version");
+	g_assert_cmpint(fwupd_report_get_created(report2), ==, 5678);
+
+	/* to JSON */
+	json1 = fwupd_report_to_json_string(report1, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(json1);
+	json2 = fwupd_report_to_json_string(report2, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(json2);
+	ret = fu_test_compare_lines(json1, json2, &error);
+	g_assert_no_error(error);
+	g_assert_true(ret);
+
+	/* to string */
+	str = fwupd_report_to_string(report2);
+	ret = fu_test_compare_lines(str,
+				    "  DeviceName:           name\n"
+				    "  DistroId:             distro_id\n"
+				    "  DistroVersion:        distro_version\n"
+				    "  VersionOld:           1.2.3\n"
+				    "  Vendor:               acme\n"
+				    "  VendorId:             2468\n"
+				    "  foo:                  bar\n"
+				    "  baz:                  bam\n",
+				    &error);
+	g_assert_no_error(error);
+	g_assert_true(ret);
 }
 
 static void
@@ -762,12 +864,21 @@ fwupd_request_func(void)
 	/* create dummy */
 	fwupd_request_set_kind(request, FWUPD_REQUEST_KIND_IMMEDIATE);
 	fwupd_request_set_id(request, FWUPD_REQUEST_ID_REMOVE_REPLUG);
+	fwupd_request_add_flag(request, FWUPD_REQUEST_FLAG_ALLOW_GENERIC_MESSAGE);
 	fwupd_request_set_message(request, "foo");
 	fwupd_request_set_image(request, "bar");
 	fwupd_request_set_device_id(request, "950da62d4c753a26e64f7f7d687104ce38e32ca5");
 	str = fwupd_request_to_string(request);
 	g_debug("%s", str);
 
+	g_assert_true(fwupd_request_has_flag(request, FWUPD_REQUEST_FLAG_ALLOW_GENERIC_MESSAGE));
+	g_assert_cmpint(fwupd_request_get_flags(request),
+			==,
+			FWUPD_REQUEST_FLAG_ALLOW_GENERIC_MESSAGE);
+	fwupd_request_remove_flag(request, FWUPD_REQUEST_FLAG_ALLOW_GENERIC_MESSAGE);
+	g_assert_false(fwupd_request_has_flag(request, FWUPD_REQUEST_FLAG_ALLOW_GENERIC_MESSAGE));
+	g_assert_cmpint(fwupd_request_get_flags(request), ==, FWUPD_REQUEST_FLAG_NONE);
+
 	/* set in init */
 	g_assert_cmpint(fwupd_request_get_created(request), >, 0);
 
@@ -983,6 +1094,7 @@ fwupd_client_devices_func(void)
 	/* only run if running fwupd is new enough */
 	ret = fwupd_client_connect(client, NULL, &error);
 	if (ret == FALSE && (g_error_matches(error, G_DBUS_ERROR, G_DBUS_ERROR_TIMED_OUT) ||
+			     g_error_matches(error, G_DBUS_ERROR, G_DBUS_ERROR_NAME_HAS_NO_OWNER) ||
 			     g_error_matches(error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN))) {
 		g_debug("%s", error->message);
 		g_test_skip("timeout connecting to daemon");
@@ -1038,6 +1150,7 @@ fwupd_client_remotes_func(void)
 	/* only run if running fwupd is new enough */
 	ret = fwupd_client_connect(client, NULL, &error);
 	if (ret == FALSE && (g_error_matches(error, G_DBUS_ERROR, G_DBUS_ERROR_TIMED_OUT) ||
+			     g_error_matches(error, G_DBUS_ERROR, G_DBUS_ERROR_NAME_HAS_NO_OWNER) ||
 			     g_error_matches(error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN))) {
 		g_debug("%s", error->message);
 		g_test_skip("timeout connecting to daemon");
@@ -1176,8 +1289,8 @@ fwupd_common_guid_func(void)
 				     &buf,
 				     FWUPD_GUID_FLAG_NONE,
 				     &error);
-	g_assert_true(ret);
 	g_assert_no_error(error);
+	g_assert_true(ret);
 	g_assert_cmpint(memcmp(buf,
 			       "\x00\x11\x22\x33\x44\x55\x66\x77\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
 			       sizeof(buf)),
@@ -1191,8 +1304,8 @@ fwupd_common_guid_func(void)
 				     &buf,
 				     FWUPD_GUID_FLAG_MIXED_ENDIAN,
 				     &error);
-	g_assert_true(ret);
 	g_assert_no_error(error);
+	g_assert_true(ret);
 	g_assert_cmpint(memcmp(buf,
 			       "\x33\x22\x11\x00\x55\x44\x77\x66\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
 			       sizeof(buf)),
@@ -1487,17 +1600,20 @@ fwupd_bios_settings_func(void)
 	g_assert_nonnull(json2);
 	ret = fu_test_compare_lines(json2,
 				    "{\n"
-				    "  \"BiosSettingReadOnly\" : \"false\",\n"
-				    "  \"BiosSettingType\" : 1,\n"
 				    "  \"Name\" : \"UEFISecureBoot\",\n"
 				    "  \"Description\" : \"Controls Secure boot\",\n"
 				    "  \"Filename\" : \"/path/to/bar\",\n"
+				    "  \"BiosSettingCurrentValue\" : \"Disabled\",\n"
+				    "  \"BiosSettingReadOnly\" : \"false\",\n"
+				    "  \"BiosSettingType\" : 1,\n"
 				    "  \"BiosSettingPossibleValues\" : [\n"
 				    "    \"Disabled\",\n"
 				    "    \"Enabled\"\n"
 				    "  ]\n"
 				    "}",
 				    &error);
+	g_assert_no_error(error);
+	g_assert_true(ret);
 }
 
 int
@@ -1516,6 +1632,7 @@ main(int argc, char **argv)
 	g_test_add_func("/fwupd/common{device-id}", fwupd_common_device_id_func);
 	g_test_add_func("/fwupd/common{guid}", fwupd_common_guid_func);
 	g_test_add_func("/fwupd/release", fwupd_release_func);
+	g_test_add_func("/fwupd/report", fwupd_report_func);
 	g_test_add_func("/fwupd/plugin", fwupd_plugin_func);
 	g_test_add_func("/fwupd/request", fwupd_request_func);
 	g_test_add_func("/fwupd/device", fwupd_device_func);
