@@ -12,6 +12,7 @@
 #include "fu-context-private.h"
 #include "fu-hwids.h"
 #include "fu-smbios-private.h"
+#include "fu-volume-private.h"
 
 /**
  * FuContext:
@@ -21,12 +22,14 @@
  */
 
 typedef struct {
+	FuContextFlags flags;
 	FuHwids *hwids;
 	FuSmbios *smbios;
 	FuQuirks *quirks;
 	GHashTable *runtime_versions;
 	GHashTable *compile_versions;
 	GPtrArray *udev_subsystems;
+	GPtrArray *esp_volumes;
 	GHashTable *firmware_gtypes;
 	GHashTable *hwid_flags; /* str: */
 	FuBatteryState battery_state;
@@ -34,6 +37,7 @@ typedef struct {
 	guint battery_level;
 	guint battery_threshold;
 	FuBiosSettings *host_bios_settings;
+	gboolean loaded_hwinfo;
 } FuContextPrivate;
 
 enum { SIGNAL_SECURITY_CHANGED, SIGNAL_LAST };
@@ -73,6 +77,10 @@ fu_context_get_smbios_string(FuContext *self, guint8 structure_type, guint8 offs
 {
 	FuContextPrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FU_IS_CONTEXT(self), NULL);
+	if (!priv->loaded_hwinfo) {
+		g_critical("cannot use SMBIOS before calling ->load_hwinfo()");
+		return NULL;
+	}
 	return fu_smbios_get_string(priv->smbios, structure_type, offset, NULL);
 }
 
@@ -80,6 +88,7 @@ fu_context_get_smbios_string(FuContext *self, guint8 structure_type, guint8 offs
  * fu_context_get_smbios_data:
  * @self: a #FuContext
  * @structure_type: a SMBIOS structure type, e.g. %FU_SMBIOS_STRUCTURE_TYPE_BIOS
+ * @error: (nullable): optional return location for an error
  *
  * Gets a hardware SMBIOS data.
  *
@@ -88,11 +97,27 @@ fu_context_get_smbios_string(FuContext *self, guint8 structure_type, guint8 offs
  * Since: 1.6.0
  **/
 GBytes *
-fu_context_get_smbios_data(FuContext *self, guint8 structure_type)
+fu_context_get_smbios_data(FuContext *self, guint8 structure_type, GError **error)
 {
 	FuContextPrivate *priv = GET_PRIVATE(self);
+	g_autoptr(GBytes) blob = NULL;
+
 	g_return_val_if_fail(FU_IS_CONTEXT(self), NULL);
-	return fu_smbios_get_data(priv->smbios, structure_type, NULL);
+
+	/* must be valid and non-zero length */
+	if (!priv->loaded_hwinfo) {
+		g_critical("cannot use SMBIOS before calling ->load_hwinfo()");
+		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_NOT_INITIALIZED, "no data");
+		return NULL;
+	}
+	blob = fu_smbios_get_data(priv->smbios, structure_type, error);
+	if (blob == NULL)
+		return NULL;
+	if (g_bytes_get_size(blob) == 0) {
+		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "no data");
+		return NULL;
+	}
+	return g_steal_pointer(&blob);
 }
 
 /**
@@ -115,6 +140,10 @@ fu_context_get_smbios_integer(FuContext *self, guint8 type, guint8 offset)
 {
 	FuContextPrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FU_IS_CONTEXT(self), G_MAXUINT);
+	if (!priv->loaded_hwinfo) {
+		g_critical("cannot use SMBIOS before calling ->load_hwinfo()");
+		return G_MAXUINT;
+	}
 	return fu_smbios_get_integer(priv->smbios, type, offset, NULL);
 }
 
@@ -209,6 +238,10 @@ fu_context_has_hwid_guid(FuContext *self, const gchar *guid)
 {
 	FuContextPrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FU_IS_CONTEXT(self), FALSE);
+	if (!priv->loaded_hwinfo) {
+		g_critical("cannot use HWIDs before calling ->load_hwinfo()");
+		return FALSE;
+	}
 	return fu_hwids_has_guid(priv->hwids, guid);
 }
 
@@ -228,6 +261,10 @@ fu_context_get_hwid_guids(FuContext *self)
 {
 	FuContextPrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FU_IS_CONTEXT(self), NULL);
+	if (!priv->loaded_hwinfo) {
+		g_critical("cannot use HWIDs before calling ->load_hwinfo()");
+		return NULL;
+	}
 	return fu_hwids_get_guids(priv->hwids);
 }
 
@@ -249,6 +286,10 @@ fu_context_get_hwid_value(FuContext *self, const gchar *key)
 	FuContextPrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FU_IS_CONTEXT(self), NULL);
 	g_return_val_if_fail(key != NULL, NULL);
+	if (!priv->loaded_hwinfo) {
+		g_critical("cannot use HWIDs before calling ->load_hwinfo()");
+		return NULL;
+	}
 	return fu_hwids_get_value(priv->hwids, key);
 }
 
@@ -271,6 +312,11 @@ fu_context_get_hwid_replace_value(FuContext *self, const gchar *keys, GError **e
 	FuContextPrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FU_IS_CONTEXT(self), NULL);
 	g_return_val_if_fail(keys != NULL, NULL);
+	if (!priv->loaded_hwinfo) {
+		g_critical("cannot use HWIDs before calling ->load_hwinfo()");
+		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_NOT_INITIALIZED, "no data");
+		return NULL;
+	}
 	return fu_hwids_get_replace_values(priv->hwids, keys, error);
 }
 
@@ -590,6 +636,7 @@ fu_context_load_hwinfo(FuContext *self, GError **error)
 	GPtrArray *guids;
 	g_autoptr(GError) error_smbios = NULL;
 	g_autoptr(GError) error_hwids = NULL;
+	g_autoptr(GError) error_bios_settings = NULL;
 
 	g_return_val_if_fail(FU_IS_CONTEXT(self), FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
@@ -598,6 +645,7 @@ fu_context_load_hwinfo(FuContext *self, GError **error)
 		g_warning("Failed to load SMBIOS: %s", error_smbios->message);
 	if (!fu_hwids_setup(priv->hwids, priv->smbios, &error_hwids))
 		g_warning("Failed to load HWIDs: %s", error_hwids->message);
+	priv->loaded_hwinfo = TRUE;
 
 	/* set the hwid flags */
 	guids = fu_context_get_hwid_guids(self);
@@ -614,8 +662,9 @@ fu_context_load_hwinfo(FuContext *self, GError **error)
 		}
 	}
 
-	/* set it up so that udev will load firmware attributes */
 	fu_context_add_udev_subsystem(self, "firmware-attributes");
+	if (!fu_context_reload_bios_settings(self, &error_bios_settings))
+		g_debug("%s", error_bios_settings->message);
 
 	/* always */
 	return TRUE;
@@ -828,6 +877,150 @@ fu_context_set_battery_threshold(FuContext *self, guint battery_threshold)
 	g_object_notify(G_OBJECT(self), "battery-threshold");
 }
 
+/**
+ * fu_context_add_flag:
+ * @context: a #FuContext
+ * @flag: the context flag
+ *
+ * Adds a specific flag to the context.
+ *
+ * Since: 1.8.5
+ **/
+void
+fu_context_add_flag(FuContext *context, FuContextFlags flag)
+{
+	FuContextPrivate *priv = GET_PRIVATE(context);
+	g_return_if_fail(FU_IS_CONTEXT(context));
+	priv->flags |= flag;
+}
+
+/**
+ * fu_context_has_flag:
+ * @context: a #FuContext
+ * @flag: the context flag
+ *
+ * Finds if the context has a specific flag.
+ *
+ * Returns: %TRUE if the flag is set
+ *
+ * Since: 1.8.5
+ **/
+gboolean
+fu_context_has_flag(FuContext *context, FuContextFlags flag)
+{
+	FuContextPrivate *priv = GET_PRIVATE(context);
+	g_return_val_if_fail(FU_IS_CONTEXT(context), FALSE);
+	return (priv->flags & flag) > 0;
+}
+
+/**
+ * fu_context_add_esp_volume:
+ * @self: a #FuContext
+ * @volume: a #FuVolume
+ *
+ * Adds an ESP volume location.
+ *
+ * Since: 1.8.5
+ **/
+void
+fu_context_add_esp_volume(FuContext *self, FuVolume *volume)
+{
+	FuContextPrivate *priv = GET_PRIVATE(self);
+
+	g_return_if_fail(FU_IS_CONTEXT(self));
+	g_return_if_fail(FU_IS_VOLUME(volume));
+
+	/* check for dupes */
+	for (guint i = 0; i < priv->esp_volumes->len; i++) {
+		FuVolume *volume_tmp = g_ptr_array_index(priv->esp_volumes, i);
+		if (g_strcmp0(fu_volume_get_id(volume_tmp), fu_volume_get_id(volume)) == 0) {
+			g_debug("not adding duplicate volume %s", fu_volume_get_id(volume));
+			return;
+		}
+	}
+
+	/* add */
+	g_ptr_array_add(priv->esp_volumes, g_object_ref(volume));
+}
+
+/**
+ * fu_context_get_esp_volumes:
+ * @self: a #FuContext
+ * @error: (nullable): optional return location for an error
+ *
+ * Finds all volumes that could be an ESP.
+ *
+ * The volumes are cached and so subsequent calls to this function will be much faster.
+ *
+ * Returns: (transfer container) (element-type FuVolume): a #GPtrArray, or %NULL if no ESP was found
+ *
+ * Since: 1.8.5
+ **/
+GPtrArray *
+fu_context_get_esp_volumes(FuContext *self, GError **error)
+{
+	FuContextPrivate *priv = GET_PRIVATE(self);
+	const gchar *path_tmp;
+	g_autoptr(GError) error_bdp = NULL;
+	g_autoptr(GError) error_esp = NULL;
+	g_autoptr(GPtrArray) volumes_bdp = NULL;
+	g_autoptr(GPtrArray) volumes_esp = NULL;
+
+	g_return_val_if_fail(FU_IS_CONTEXT(self), NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+
+	/* cached result */
+	if (priv->esp_volumes->len > 0)
+		return g_ptr_array_ref(priv->esp_volumes);
+
+	/* for the test suite use local directory for ESP */
+	path_tmp = g_getenv("FWUPD_UEFI_ESP_PATH");
+	if (path_tmp != NULL) {
+		g_autoptr(FuVolume) vol = fu_volume_new_from_mount_path(path_tmp);
+		fu_context_add_esp_volume(self, vol);
+		return g_ptr_array_ref(priv->esp_volumes);
+	}
+
+	/* ESP */
+	volumes_esp = fu_volume_new_by_kind(FU_VOLUME_KIND_ESP, &error_esp);
+	if (volumes_esp == NULL) {
+		g_debug("%s", error_esp->message);
+	} else {
+		for (guint i = 0; i < volumes_esp->len; i++) {
+			FuVolume *vol = g_ptr_array_index(volumes_esp, i);
+			g_autofree gchar *type = fu_volume_get_id_type(vol);
+			if (g_strcmp0(type, "ext4") == 0)
+				continue;
+			fu_context_add_esp_volume(self, vol);
+		}
+	}
+
+	/* BDP */
+	volumes_bdp = fu_volume_new_by_kind(FU_VOLUME_KIND_BDP, &error_bdp);
+	if (volumes_bdp == NULL) {
+		g_debug("%s", error_bdp->message);
+	} else {
+		for (guint i = 0; i < volumes_bdp->len; i++) {
+			FuVolume *vol = g_ptr_array_index(volumes_bdp, i);
+			g_autofree gchar *type = fu_volume_get_id_type(vol);
+			if (g_strcmp0(type, "vfat") != 0)
+				continue;
+			if (!fu_volume_is_internal(vol))
+				continue;
+			fu_context_add_esp_volume(self, vol);
+		}
+	}
+
+	/* nothing found */
+	if (priv->esp_volumes->len == 0) {
+		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND, "No ESP or BDP found");
+		return NULL;
+	}
+
+	/* success */
+	return g_ptr_array_ref(priv->esp_volumes);
+}
+
 static void
 fu_context_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
@@ -892,6 +1085,7 @@ fu_context_finalize(GObject *object)
 	g_object_unref(priv->host_bios_settings);
 	g_hash_table_unref(priv->firmware_gtypes);
 	g_ptr_array_unref(priv->udev_subsystems);
+	g_ptr_array_unref(priv->esp_volumes);
 
 	G_OBJECT_CLASS(fu_context_parent_class)->finalize(object);
 }
@@ -1005,6 +1199,7 @@ fu_context_init(FuContext *self)
 	priv->firmware_gtypes = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
 	priv->quirks = fu_quirks_new();
 	priv->host_bios_settings = fu_bios_settings_new();
+	priv->esp_volumes = g_ptr_array_new_with_free_func((GDestroyNotify)g_object_unref);
 }
 
 /**
