@@ -9,6 +9,7 @@
 #include "config.h"
 
 #include <errno.h>
+#include <fcntl.h>
 #include <fwupd.h>
 #include <glib/gstdio.h>
 #include <gmodule.h>
@@ -55,6 +56,8 @@ typedef struct {
 	FuPluginVfuncs vfuncs;
 } FuPluginPrivate;
 
+enum { PROP_0, PROP_CONTEXT, PROP_LAST };
+
 enum {
 	SIGNAL_DEVICE_ADDED,
 	SIGNAL_DEVICE_REMOVED,
@@ -83,6 +86,9 @@ typedef gboolean (*FuPluginFlaggedDeviceFunc)(FuPlugin *self,
 					      GError **error);
 typedef gboolean (*FuPluginDeviceArrayFunc)(FuPlugin *self, GPtrArray *devices, GError **error);
 
+#define FU_PLUGIN_FILE_MODE_NONSECURE 0644
+#define FU_PLUGIN_FILE_MODE_SECURE    0640
+
 /**
  * fu_plugin_is_open:
  * @self: a #FuPlugin
@@ -137,25 +143,9 @@ static FuPluginVfuncs *
 fu_plugin_get_vfuncs(FuPlugin *self)
 {
 	FuPluginPrivate *priv = GET_PRIVATE(self);
-	return &priv->vfuncs;
-}
-
-/**
- * fu_plugin_get_build_hash:
- * @self: a #FuPlugin
- *
- * Gets the build hash a plugin was generated with.
- *
- * Returns: (transfer none): a #gchar, or %NULL for unset.
- *
- * Since: 1.2.4
- **/
-const gchar *
-fu_plugin_get_build_hash(FuPlugin *self)
-{
-	FuPluginVfuncs *vfuncs = fu_plugin_get_vfuncs(self);
-	g_return_val_if_fail(FU_IS_PLUGIN(self), NULL);
-	return vfuncs->build_hash;
+	if (fu_plugin_has_flag(self, FWUPD_PLUGIN_FLAG_MODULAR))
+		return &priv->vfuncs;
+	return FU_PLUGIN_GET_CLASS(self);
 }
 
 /**
@@ -298,6 +288,14 @@ fu_plugin_guess_name_from_fn(const gchar *filename)
 	return name;
 }
 
+static gchar *
+fu_plugin_get_config_filename(FuPlugin *self)
+{
+	g_autofree gchar *conf_dir = fu_path_from_kind(FU_PATH_KIND_SYSCONFDIR_PKG);
+	g_autofree gchar *conf_file = g_strdup_printf("%s.conf", fu_plugin_get_name(self));
+	return g_build_filename(conf_dir, conf_file, NULL);
+}
+
 /**
  * fu_plugin_open:
  * @self: a #FuPlugin
@@ -314,7 +312,7 @@ gboolean
 fu_plugin_open(FuPlugin *self, const gchar *filename, GError **error)
 {
 	FuPluginPrivate *priv = GET_PRIVATE(self);
-	FuPluginVfuncs *vfuncs = fu_plugin_get_vfuncs(self);
+	FuPluginVfuncs *vfuncs;
 	FuPluginInitVfuncsFunc init_vfuncs = NULL;
 
 	g_return_val_if_fail(FU_IS_PLUGIN(self), FALSE);
@@ -346,6 +344,10 @@ fu_plugin_open(FuPlugin *self, const gchar *filename, GError **error)
 		fu_plugin_add_flag(self, FWUPD_PLUGIN_FLAG_USER_WARNING);
 		return FALSE;
 	}
+
+	/* we can't "fallback" from modular to built-in so this is safe */
+	fu_plugin_add_flag(self, FWUPD_PLUGIN_FLAG_MODULAR);
+	vfuncs = fu_plugin_get_vfuncs(self);
 	init_vfuncs(vfuncs);
 
 	/* set automatically */
@@ -501,14 +503,6 @@ fu_plugin_config_monitor_changed_cb(GFileMonitor *monitor,
 	g_signal_emit(self, signals[SIGNAL_CONFIG_CHANGED], 0);
 }
 
-static gchar *
-fu_plugin_get_config_filename(FuPlugin *self)
-{
-	g_autofree gchar *conf_dir = fu_path_from_kind(FU_PATH_KIND_SYSCONFDIR_PKG);
-	g_autofree gchar *conf_file = g_strdup_printf("%s.conf", fu_plugin_get_name(self));
-	return g_build_filename(conf_dir, conf_file, NULL);
-}
-
 /**
  * fu_plugin_device_add:
  * @self: a #FuPlugin
@@ -693,6 +687,26 @@ fu_plugin_get_context(FuPlugin *self)
 	return priv->ctx;
 }
 
+/**
+ * fu_plugin_set_context:
+ * @self: a #FuPlugin
+ * @ctx: (nullable): optional #FuContext
+ *
+ * Sets the context for this plugin.
+ *
+ * Since: 1.8.6
+ **/
+void
+fu_plugin_set_context(FuPlugin *self, FuContext *ctx)
+{
+	FuPluginPrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(FU_IS_PLUGIN(self));
+	g_return_if_fail(FU_IS_CONTEXT(ctx) || ctx == NULL);
+
+	if (g_set_object(&priv->ctx, ctx))
+		g_object_notify(G_OBJECT(self), "context");
+}
+
 static gboolean
 fu_plugin_device_attach(FuPlugin *self, FuDevice *device, FuProgress *progress, GError **error)
 {
@@ -881,23 +895,40 @@ fu_plugin_runner_startup(FuPlugin *self, FuProgress *progress, GError **error)
 	if (fu_plugin_has_flag(self, FWUPD_PLUGIN_FLAG_DISABLED))
 		return TRUE;
 
-	/* optional */
-	if (vfuncs->startup == NULL)
-		return TRUE;
-	g_debug("startup(%s)", fu_plugin_get_name(self));
-	if (!vfuncs->startup(self, progress, &error_local)) {
-		if (error_local == NULL) {
-			g_critical("unset plugin error in startup(%s)", fu_plugin_get_name(self));
-			g_set_error_literal(&error_local,
-					    FWUPD_ERROR,
-					    FWUPD_ERROR_INTERNAL,
-					    "unspecified error");
+	/* ensure the configure file is set to the correct permission */
+	if (fu_plugin_has_flag(self, FWUPD_PLUGIN_FLAG_SECURE_CONFIG)) {
+		if (g_file_test(config_filename, G_FILE_TEST_EXISTS)) {
+			gint rc = g_chmod(config_filename, FU_PLUGIN_FILE_MODE_SECURE);
+			if (rc != 0) {
+				g_set_error(error,
+					    G_IO_ERROR,
+					    G_IO_ERROR_FAILED,
+					    "failed to change permission of %s",
+					    config_filename);
+				fu_plugin_add_flag(self, FWUPD_PLUGIN_FLAG_FAILED_OPEN);
+				return FALSE;
+			}
 		}
-		g_propagate_prefixed_error(error,
-					   g_steal_pointer(&error_local),
-					   "failed to startup using %s: ",
+	}
+
+	/* optional */
+	if (vfuncs->startup != NULL) {
+		g_debug("startup(%s)", fu_plugin_get_name(self));
+		if (!vfuncs->startup(self, progress, &error_local)) {
+			if (error_local == NULL) {
+				g_critical("unset plugin error in startup(%s)",
 					   fu_plugin_get_name(self));
-		return FALSE;
+				g_set_error_literal(&error_local,
+						    FWUPD_ERROR,
+						    FWUPD_ERROR_INTERNAL,
+						    "unspecified error");
+			}
+			g_propagate_prefixed_error(error,
+						   g_steal_pointer(&error_local),
+						   "failed to startup using %s: ",
+						   fu_plugin_get_name(self));
+			return FALSE;
+		}
 	}
 
 	/* create a monitor on the config file */
@@ -917,7 +948,7 @@ fu_plugin_runner_startup(FuPlugin *self, FuProgress *progress, GError **error)
  * fu_plugin_runner_init:
  * @self: a #FuPlugin
  *
- * Runs the init routine for the plugin, if enabled.
+ * Runs the constructed routine for the plugin, if enabled.
  *
  * Since: 1.8.1
  **/
@@ -938,9 +969,9 @@ fu_plugin_runner_init(FuPlugin *self)
 		return;
 
 	/* optional */
-	if (vfuncs->init != NULL) {
-		g_debug("init(%s)", fu_plugin_get_name(self));
-		vfuncs->init(self);
+	if (vfuncs->constructed != NULL) {
+		g_debug("constructed(%s)", fu_plugin_get_name(self));
+		vfuncs->constructed(G_OBJECT(self));
 		priv->done_init = TRUE;
 	}
 }
@@ -1226,6 +1257,7 @@ fu_plugin_runner_prepare(FuPlugin *self,
 			 GError **error)
 {
 	FuPluginVfuncs *vfuncs = fu_plugin_get_vfuncs(self);
+	fu_device_add_backend_tag(device, "prepare");
 	return fu_plugin_runner_flagged_device_generic(self,
 						       device,
 						       progress,
@@ -1257,6 +1289,7 @@ fu_plugin_runner_cleanup(FuPlugin *self,
 			 GError **error)
 {
 	FuPluginVfuncs *vfuncs = fu_plugin_get_vfuncs(self);
+	fu_device_add_backend_tag(device, "cleanup");
 	return fu_plugin_runner_flagged_device_generic(self,
 						       device,
 						       progress,
@@ -1283,6 +1316,7 @@ gboolean
 fu_plugin_runner_attach(FuPlugin *self, FuDevice *device, FuProgress *progress, GError **error)
 {
 	FuPluginVfuncs *vfuncs = fu_plugin_get_vfuncs(self);
+	fu_device_add_backend_tag(device, "attach");
 	return fu_plugin_runner_device_generic_progress(
 	    self,
 	    device,
@@ -1309,6 +1343,7 @@ gboolean
 fu_plugin_runner_detach(FuPlugin *self, FuDevice *device, FuProgress *progress, GError **error)
 {
 	FuPluginVfuncs *vfuncs = fu_plugin_get_vfuncs(self);
+	fu_device_add_backend_tag(device, "detach");
 	return fu_plugin_runner_device_generic_progress(
 	    self,
 	    device,
@@ -1344,6 +1379,7 @@ fu_plugin_runner_reload(FuPlugin *self, FuDevice *device, GError **error)
 	locker = fu_device_locker_new(proxy, error);
 	if (locker == NULL)
 		return FALSE;
+	fu_device_add_backend_tag(device, "reload");
 	return fu_device_reload(device, error);
 }
 
@@ -1511,7 +1547,10 @@ fu_plugin_check_supported_device(FuPlugin *self, FuDevice *device)
 }
 
 static gboolean
-fu_plugin_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
+fu_plugin_backend_device_added(FuPlugin *self,
+			       FuDevice *device,
+			       FuProgress *progress,
+			       GError **error)
 {
 	FuDevice *proxy;
 	FuPluginPrivate *priv = GET_PRIVATE(self);
@@ -1519,6 +1558,13 @@ fu_plugin_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
 	g_autoptr(FuDevice) dev = NULL;
 	g_autoptr(FuDeviceLocker) locker = NULL;
 
+	/* progress */
+	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_NO_PROFILE);
+	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 4, "created");
+	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 48, "open");
+	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 48, "add");
+
 	/* fall back to plugin default */
 	if (device_gtype == G_TYPE_INVALID) {
 		if (priv->device_gtypes->len > 1) {
@@ -1536,6 +1582,7 @@ fu_plugin_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
 	fu_device_incorporate(dev, FU_DEVICE(device));
 	if (!fu_plugin_runner_device_created(self, dev, error))
 		return FALSE;
+	fu_progress_step_done(progress);
 
 	/* there are a lot of different devices that match, but not all respond
 	 * well to opening -- so limit some ones with issued updates */
@@ -1546,11 +1593,12 @@ fu_plugin_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
 		if (!fu_plugin_check_supported_device(self, dev)) {
 			g_autofree gchar *guids = fu_device_get_guids_as_str(dev);
 			g_debug("%s has no updates, so ignoring device", guids);
+			fu_progress_finished(progress);
 			return TRUE;
 		}
 	}
 
-	/* open and add */
+	/* open */
 	proxy = fu_device_get_proxy(device);
 	if (proxy != NULL) {
 		g_autoptr(FuDeviceLocker) locker_proxy = NULL;
@@ -1561,8 +1609,12 @@ fu_plugin_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
 	locker = fu_device_locker_new(dev, error);
 	if (locker == NULL)
 		return FALSE;
+	fu_progress_step_done(progress);
+
+	/* add */
 	fu_plugin_device_add(self, dev);
 	fu_plugin_runner_device_added(self, dev);
+	fu_progress_step_done(progress);
 	return TRUE;
 }
 
@@ -1570,6 +1622,7 @@ fu_plugin_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
  * fu_plugin_runner_backend_device_added:
  * @self: a #FuPlugin
  * @device: a device
+ * @progress: a #FuProgress
  * @error: (nullable): optional return location for an error
  *
  * Call the backend_device_added routine for the plugin
@@ -1579,7 +1632,10 @@ fu_plugin_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
  * Since: 1.5.6
  **/
 gboolean
-fu_plugin_runner_backend_device_added(FuPlugin *self, FuDevice *device, GError **error)
+fu_plugin_runner_backend_device_added(FuPlugin *self,
+				      FuDevice *device,
+				      FuProgress *progress,
+				      GError **error)
 {
 	FuPluginPrivate *priv = GET_PRIVATE(self);
 	FuPluginVfuncs *vfuncs = fu_plugin_get_vfuncs(self);
@@ -1587,6 +1643,7 @@ fu_plugin_runner_backend_device_added(FuPlugin *self, FuDevice *device, GError *
 
 	g_return_val_if_fail(FU_IS_PLUGIN(self), FALSE);
 	g_return_val_if_fail(FU_IS_DEVICE(device), FALSE);
+	g_return_val_if_fail(FU_IS_PROGRESS(progress), FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
 	/* not enabled */
@@ -1597,7 +1654,7 @@ fu_plugin_runner_backend_device_added(FuPlugin *self, FuDevice *device, GError *
 	if (vfuncs->backend_device_added == NULL) {
 		if (priv->device_gtypes != NULL ||
 		    fu_device_get_specialized_gtype(device) != G_TYPE_INVALID) {
-			return fu_plugin_backend_device_added(self, device, error);
+			return fu_plugin_backend_device_added(self, device, progress, error);
 		}
 		g_set_error_literal(error,
 				    FWUPD_ERROR,
@@ -1917,6 +1974,7 @@ fu_plugin_runner_activate(FuPlugin *self, FuDevice *device, FuProgress *progress
 	}
 
 	/* run vfunc */
+	fu_device_add_backend_tag(device, "activate");
 	if (!fu_plugin_runner_device_generic_progress(
 		self,
 		device,
@@ -1966,6 +2024,7 @@ fu_plugin_runner_unlock(FuPlugin *self, FuDevice *device, GError **error)
 	}
 
 	/* run vfunc */
+	fu_device_add_backend_tag(device, "unlock");
 	if (!fu_plugin_runner_device_generic(self,
 					     device,
 					     "fu_plugin_unlock",
@@ -2015,6 +2074,7 @@ fu_plugin_runner_write_firmware(FuPlugin *self,
 		g_debug("plugin not enabled, skipping");
 		return TRUE;
 	}
+	fu_device_add_backend_tag(device, "write-firmware");
 
 	/* optional */
 	if (vfuncs->write_firmware == NULL) {
@@ -2379,6 +2439,78 @@ fu_plugin_security_attr_new(FuPlugin *self, const gchar *appstream_id)
 	return g_steal_pointer(&attr);
 }
 
+#if !GLIB_CHECK_VERSION(2, 66, 0)
+
+#define G_FILE_SET_CONTENTS_CONSISTENT 0
+typedef guint GFileSetContentsFlags;
+static gboolean
+g_file_set_contents_full(const gchar *filename,
+			 const gchar *contents,
+			 gssize length,
+			 GFileSetContentsFlags flags,
+			 int mode,
+			 GError **error)
+{
+	gint fd;
+	gssize wrote;
+
+	if (length < 0)
+		length = strlen(contents);
+	fd = g_open(filename, O_CREAT, mode);
+	if (fd <= 0) {
+		g_set_error(error,
+			    G_IO_ERROR,
+			    G_IO_ERROR_FAILED,
+			    "could not open %s file",
+			    filename);
+		return FALSE;
+	}
+	wrote = write(fd, contents, length);
+	if (wrote != length) {
+		g_set_error(error,
+			    G_IO_ERROR,
+			    G_IO_ERROR_FAILED,
+			    "did not write %s file",
+			    filename);
+		g_close(fd, NULL);
+		return FALSE;
+	}
+	return g_close(fd, error);
+}
+#endif
+
+static gboolean
+fu_plugin_set_config_value_internal(FuPlugin *self,
+				    const gchar *key,
+				    const gchar *value,
+				    guint32 mode,
+				    GError **error)
+{
+	g_autofree gchar *conf_path = fu_plugin_get_config_filename(self);
+	g_autofree gchar *data = NULL;
+	g_autoptr(GKeyFile) keyfile = g_key_file_new();
+
+	g_return_val_if_fail(FU_IS_PLUGIN(self), FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	if (!g_file_test(conf_path, G_FILE_TEST_EXISTS)) {
+		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND, "%s is missing", conf_path);
+		return FALSE;
+	}
+	if (!g_key_file_load_from_file(keyfile, conf_path, G_KEY_FILE_KEEP_COMMENTS, error))
+		return FALSE;
+	g_key_file_set_string(keyfile, fu_plugin_get_name(self), key, value);
+	data = g_key_file_to_data(keyfile, NULL, error);
+	if (data == NULL)
+		return FALSE;
+	return g_file_set_contents_full(conf_path,
+					data,
+					-1,
+					G_FILE_SET_CONTENTS_CONSISTENT,
+					mode,
+					error);
+}
+
 /**
  * fu_plugin_set_config_value:
  * @self: a #FuPlugin
@@ -2395,61 +2527,23 @@ fu_plugin_security_attr_new(FuPlugin *self, const gchar *appstream_id)
 gboolean
 fu_plugin_set_config_value(FuPlugin *self, const gchar *key, const gchar *value, GError **error)
 {
-	g_autofree gchar *conf_path = fu_plugin_get_config_filename(self);
-	g_autoptr(GKeyFile) keyfile = NULL;
-
 	g_return_val_if_fail(FU_IS_PLUGIN(self), FALSE);
 	g_return_val_if_fail(key != NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	keyfile = g_key_file_new();
-	if (!g_key_file_load_from_file(keyfile, conf_path, G_KEY_FILE_KEEP_COMMENTS, error))
-		return FALSE;
-	g_key_file_set_string(keyfile, fu_plugin_get_name(self), key, value);
-	return g_key_file_save_to_file(keyfile, conf_path, error);
-}
-
-/**
- * fu_plugin_set_secure_config_value:
- * @self: a #FuPlugin
- * @key: a settings key
- * @value: (nullable): a settings value
- * @error: (nullable): optional return location for an error
- *
- * Sets a plugin config file value and updates file so that non-privileged users
- * cannot read it.
- *
- * Returns: %TRUE for success
- *
- * Since: 1.7.4
- **/
-gboolean
-fu_plugin_set_secure_config_value(FuPlugin *self,
-				  const gchar *key,
-				  const gchar *value,
-				  GError **error)
-{
-	g_autofree gchar *conf_path = fu_plugin_get_config_filename(self);
-	gint ret;
-
-	g_return_val_if_fail(FU_IS_PLUGIN(self), FALSE);
-	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
-
-	if (!g_file_test(conf_path, G_FILE_TEST_EXISTS)) {
-		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND, "%s is missing", conf_path);
-		return FALSE;
+	/* when removing fu_plugin_set_secure_config_value() just remove this instead */
+	if (fu_plugin_has_flag(self, FWUPD_PLUGIN_FLAG_SECURE_CONFIG)) {
+		return fu_plugin_set_config_value_internal(self,
+							   key,
+							   value,
+							   FU_PLUGIN_FILE_MODE_SECURE,
+							   error);
 	}
-	ret = g_chmod(conf_path, 0660);
-	if (ret == -1) {
-		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_INTERNAL,
-			    "failed to set permissions on %s",
-			    conf_path);
-		return FALSE;
-	}
-
-	return fu_plugin_set_config_value(self, key, value, error);
+	return fu_plugin_set_config_value_internal(self,
+						   key,
+						   value,
+						   FU_PLUGIN_FILE_MODE_NONSECURE,
+						   error);
 }
 
 /**
@@ -2512,11 +2606,74 @@ fu_plugin_order_compare(FuPlugin *plugin1, FuPlugin *plugin2)
 	return fu_plugin_name_compare(plugin1, plugin2);
 }
 
+static gchar *
+fu_plugin_convert_gtype_to_name(GType gtype)
+{
+	const gchar *gtype_name = g_type_name(gtype);
+	gsize len = strlen(gtype_name);
+	g_autoptr(GString) str = g_string_new(NULL);
+
+	g_return_val_if_fail(g_str_has_prefix(gtype_name, "Fu"), NULL);
+	g_return_val_if_fail(g_str_has_suffix(gtype_name, "Plugin"), NULL);
+
+	/* self tests */
+	if (g_strcmp0(gtype_name, "FuPlugin") == 0)
+		return g_strdup("plugin");
+
+	/* normal plugins */
+	for (guint j = 2; j < len - 6; j++) {
+		gchar tmp = gtype_name[j];
+		if (g_ascii_isupper(tmp)) {
+			if (str->len > 0)
+				g_string_append_c(str, '_');
+			g_string_append_c(str, g_ascii_tolower(tmp));
+		} else {
+			g_string_append_c(str, tmp);
+		}
+	}
+	if (str->len == 0)
+		return NULL;
+	return g_string_free(g_steal_pointer(&str), FALSE);
+}
+
+static void
+fu_plugin_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	FuPlugin *self = FU_PLUGIN(object);
+	FuPluginPrivate *priv = GET_PRIVATE(self);
+	switch (prop_id) {
+	case PROP_CONTEXT:
+		g_value_set_object(value, priv->ctx);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+fu_plugin_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+	FuPlugin *self = FU_PLUGIN(object);
+	switch (prop_id) {
+	case PROP_CONTEXT:
+		fu_plugin_set_context(self, g_value_get_object(value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 fu_plugin_class_init(FuPluginClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
+	GParamSpec *pspec;
+
 	object_class->finalize = fu_plugin_finalize;
+	object_class->get_property = fu_plugin_get_property;
+	object_class->set_property = fu_plugin_set_property;
 
 	/**
 	 * FuPlugin::device-added:
@@ -2530,7 +2687,7 @@ fu_plugin_class_init(FuPluginClass *klass)
 	signals[SIGNAL_DEVICE_ADDED] = g_signal_new("device-added",
 						    G_TYPE_FROM_CLASS(object_class),
 						    G_SIGNAL_RUN_LAST,
-						    G_STRUCT_OFFSET(FuPluginClass, device_added),
+						    G_STRUCT_OFFSET(FuPluginClass, _device_added),
 						    NULL,
 						    NULL,
 						    g_cclosure_marshal_VOID__OBJECT,
@@ -2550,7 +2707,7 @@ fu_plugin_class_init(FuPluginClass *klass)
 	    g_signal_new("device-removed",
 			 G_TYPE_FROM_CLASS(object_class),
 			 G_SIGNAL_RUN_LAST,
-			 G_STRUCT_OFFSET(FuPluginClass, device_removed),
+			 G_STRUCT_OFFSET(FuPluginClass, _device_removed),
 			 NULL,
 			 NULL,
 			 g_cclosure_marshal_VOID__OBJECT,
@@ -2570,7 +2727,7 @@ fu_plugin_class_init(FuPluginClass *klass)
 	    g_signal_new("device-register",
 			 G_TYPE_FROM_CLASS(object_class),
 			 G_SIGNAL_RUN_LAST,
-			 G_STRUCT_OFFSET(FuPluginClass, device_register),
+			 G_STRUCT_OFFSET(FuPluginClass, _device_register),
 			 NULL,
 			 NULL,
 			 g_cclosure_marshal_VOID__OBJECT,
@@ -2593,7 +2750,7 @@ fu_plugin_class_init(FuPluginClass *klass)
 	    g_signal_new("check-supported",
 			 G_TYPE_FROM_CLASS(object_class),
 			 G_SIGNAL_RUN_LAST,
-			 G_STRUCT_OFFSET(FuPluginClass, check_supported),
+			 G_STRUCT_OFFSET(FuPluginClass, _check_supported),
 			 NULL,
 			 NULL,
 			 g_cclosure_marshal_generic,
@@ -2603,7 +2760,7 @@ fu_plugin_class_init(FuPluginClass *klass)
 	signals[SIGNAL_RULES_CHANGED] = g_signal_new("rules-changed",
 						     G_TYPE_FROM_CLASS(object_class),
 						     G_SIGNAL_RUN_LAST,
-						     G_STRUCT_OFFSET(FuPluginClass, rules_changed),
+						     G_STRUCT_OFFSET(FuPluginClass, _rules_changed),
 						     NULL,
 						     NULL,
 						     g_cclosure_marshal_VOID__VOID,
@@ -2622,12 +2779,26 @@ fu_plugin_class_init(FuPluginClass *klass)
 	    g_signal_new("config-changed",
 			 G_TYPE_FROM_CLASS(object_class),
 			 G_SIGNAL_RUN_LAST,
-			 G_STRUCT_OFFSET(FuPluginClass, config_changed),
+			 G_STRUCT_OFFSET(FuPluginClass, _config_changed),
 			 NULL,
 			 NULL,
 			 g_cclosure_marshal_VOID__VOID,
 			 G_TYPE_NONE,
 			 0);
+
+	/**
+	 * FuPlugin:context:
+	 *
+	 * The #FuContext to use.
+	 *
+	 * Since: 1.8.6
+	 */
+	pspec = g_param_spec_object("context",
+				    NULL,
+				    NULL,
+				    FU_TYPE_CONTEXT,
+				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME);
+	g_object_class_install_property(object_class, PROP_CONTEXT, pspec);
 }
 
 static void
@@ -2647,9 +2818,9 @@ fu_plugin_finalize(GObject *object)
 	g_rw_lock_clear(&priv->cache_mutex);
 
 	/* optional */
-	if (priv->done_init && vfuncs->destroy != NULL) {
-		g_debug("destroy(%s)", fu_plugin_get_name(self));
-		vfuncs->destroy(self);
+	if (priv->done_init && vfuncs->finalize != NULL) {
+		g_debug("finalize(%s)", fu_plugin_get_name(self));
+		vfuncs->finalize(G_OBJECT(self));
 	}
 
 	for (guint i = 0; i < FU_PLUGIN_RULE_LAST; i++) {
@@ -2677,6 +2848,31 @@ fu_plugin_finalize(GObject *object)
 	G_OBJECT_CLASS(fu_plugin_parent_class)->finalize(object);
 }
 
+/**
+ * fu_plugin_new_from_gtype:
+ * @ctx: (nullable): a #FuContext
+ * @gtype: a #GType, possibly even `G_TYPE_PLUGIN`
+ *
+ * Creates a new #FuPlugin
+ *
+ * Since: 1.8.6
+ **/
+FuPlugin *
+fu_plugin_new_from_gtype(GType gtype, FuContext *ctx)
+{
+	FuPlugin *self;
+
+	g_return_val_if_fail(gtype != G_TYPE_INVALID, NULL);
+	g_return_val_if_fail(ctx == NULL || FU_IS_CONTEXT(ctx), NULL);
+
+	self = g_object_new(gtype, "context", ctx, NULL);
+	if (fu_plugin_get_name(self) == NULL) {
+		g_autofree gchar *name = fu_plugin_convert_gtype_to_name(gtype);
+		fu_plugin_set_name(self, name);
+	}
+	return self;
+}
+
 /**
  * fu_plugin_new:
  * @ctx: (nullable): a #FuContext
@@ -2689,8 +2885,7 @@ FuPlugin *
 fu_plugin_new(FuContext *ctx)
 {
 	FuPlugin *self = FU_PLUGIN(g_object_new(FU_TYPE_PLUGIN, NULL));
-	FuPluginPrivate *priv = GET_PRIVATE(self);
 	if (ctx != NULL)
-		priv->ctx = g_object_ref(ctx);
+		fu_plugin_set_context(self, ctx);
 	return self;
 }
