@@ -22,11 +22,6 @@
 #include "fu-quirks.h"
 #include "fu-string.h"
 
-#ifdef _WIN32
-#include <fwupd-windows.h>
-#include <windows.h>
-#endif
-
 /**
  * FuQuirks:
  *
@@ -124,79 +119,128 @@ fu_quirks_validate_flags(const gchar *value, GError **error)
 	return TRUE;
 }
 
+typedef struct {
+	GString *group;
+	XbBuilderNode *bn;
+	XbBuilderNode *root;
+} FuQuirksConvertHelper;
+
+static void
+fu_quirks_convert_helper_free(FuQuirksConvertHelper *helper)
+{
+	g_string_free(helper->group, TRUE);
+	g_object_unref(helper->root);
+	if (helper->bn != NULL)
+		g_object_unref(helper->bn);
+	g_free(helper);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(FuQuirksConvertHelper, fu_quirks_convert_helper_free)
+
+static gboolean
+fu_quirks_convert_keyfile_to_xml_cb(GString *token,
+				    guint token_idx,
+				    gpointer user_data,
+				    GError **error)
+{
+	FuQuirksConvertHelper *helper = (FuQuirksConvertHelper *)user_data;
+	g_autofree gchar *key = NULL;
+	g_autofree gchar *value = NULL;
+	g_auto(GStrv) kv = NULL;
+
+	/* blank line */
+	if (token->len == 0)
+		return TRUE;
+
+	/* comment */
+	if (token->str[0] == '#')
+		return TRUE;
+
+	/* neither a key=value or [group] */
+	if (token->len < 3) {
+		g_set_error(error,
+			    G_IO_ERROR,
+			    G_IO_ERROR_INVALID_DATA,
+			    "invalid line: %s",
+			    token->str);
+		return FALSE;
+	}
+
+	/* a group */
+	if (token->str[0] == '[' && token->str[token->len - 1] == ']') {
+		g_autofree gchar *group_id = NULL;
+		g_autofree gchar *group_tmp = NULL;
+		g_autoptr(XbBuilderNode) bn_tmp = NULL;
+
+		/* trim off the [] and convert to a GUID */
+		group_tmp = g_strndup(token->str + 1, token->len - 2);
+		group_id = fu_quirks_build_group_key(group_tmp);
+		bn_tmp = xb_builder_node_insert(helper->root, "device", "id", group_id, NULL);
+		g_set_object(&helper->bn, bn_tmp);
+		g_string_assign(helper->group, group_tmp);
+		return TRUE;
+	}
+
+	/* no current group */
+	if (helper->bn == NULL) {
+		g_set_error(error,
+			    G_IO_ERROR,
+			    G_IO_ERROR_INVALID_DATA,
+			    "invalid line when group unset: %s",
+			    token->str);
+		return FALSE;
+	}
+
+	/* parse as key=value */
+	kv = g_strsplit(token->str, "=", 2);
+	if (kv[1] == NULL) {
+		g_set_error(error,
+			    G_IO_ERROR,
+			    G_IO_ERROR_INVALID_DATA,
+			    "invalid line: not key=value: %s",
+			    token->str);
+		return FALSE;
+	}
+
+	/* sanity check flags */
+	key = fu_strstrip(kv[0]);
+	value = fu_strstrip(kv[1]);
+	if (g_strcmp0(key, FU_QUIRKS_FLAGS) == 0) {
+		g_autoptr(GError) error_local = NULL;
+		if (!fu_quirks_validate_flags(value, &error_local)) {
+			g_warning("[%s] %s = %s is invalid: %s",
+				  helper->group->str,
+				  key,
+				  value,
+				  error_local->message);
+		}
+	}
+
+	/* add */
+	xb_builder_node_insert_text(helper->bn, "value", value, "key", key, NULL);
+	return TRUE;
+}
+
 static GBytes *
 fu_quirks_convert_keyfile_to_xml(FuQuirks *self, GBytes *bytes, GError **error)
 {
 	gsize xmlsz;
 	g_autofree gchar *xml = NULL;
-	g_auto(GStrv) groups = NULL;
-	g_autoptr(GKeyFile) kf = g_key_file_new();
-	g_autoptr(XbBuilderNode) root = xb_builder_node_new("quirk");
+	g_autoptr(FuQuirksConvertHelper) helper = g_new0(FuQuirksConvertHelper, 1);
 
-	/* parse keyfile */
-	if (!g_key_file_load_from_data(kf,
-				       g_bytes_get_data(bytes, NULL),
-				       g_bytes_get_size(bytes),
-				       G_KEY_FILE_NONE,
-				       error))
+	/* split into lines */
+	helper->root = xb_builder_node_new("quirk");
+	helper->group = g_string_new(NULL);
+	if (!fu_strsplit_full((const gchar *)g_bytes_get_data(bytes, NULL),
+			      g_bytes_get_size(bytes),
+			      "\n",
+			      fu_quirks_convert_keyfile_to_xml_cb,
+			      helper,
+			      error))
 		return NULL;
 
-	/* add each set of groups and keys */
-	groups = g_key_file_get_groups(kf, NULL);
-	for (guint i = 0; groups[i] != NULL; i++) {
-		g_auto(GStrv) keys = NULL;
-		g_autofree gchar *group_id = NULL;
-		g_autoptr(GError) error_local = NULL;
-		g_autoptr(XbBuilderNode) bn = NULL;
-
-		/* sanity check group */
-		if (g_str_has_prefix(groups[i], "HwID") ||
-		    g_str_has_prefix(groups[i], "DeviceInstanceID") ||
-		    g_str_has_prefix(groups[i], "GUID")) {
-			g_warning("invalid group name '%s'", groups[i]);
-			continue;
-		}
-
-		/* get all KVs for the entry */
-		keys = g_key_file_get_keys(kf, groups[i], NULL, error);
-		if (keys == NULL)
-			return NULL;
-		group_id = fu_quirks_build_group_key(groups[i]);
-		bn = xb_builder_node_insert(root, "device", "id", group_id, NULL);
-		for (guint j = 0; keys[j] != NULL; j++) {
-			g_autofree gchar *value = NULL;
-
-			/* sanity check key */
-			if ((self->load_flags & FU_QUIRKS_LOAD_FLAG_NO_VERIFY) == 0 &&
-			    g_hash_table_lookup(self->possible_keys, keys[j]) == NULL) {
-				if (!g_ptr_array_find_with_equal_func(self->invalid_keys,
-								      keys[j],
-								      g_str_equal,
-								      NULL)) {
-					g_ptr_array_add(self->invalid_keys, g_strdup(keys[j]));
-				}
-			}
-			value = g_key_file_get_value(kf, groups[i], keys[j], error);
-			if (value == NULL)
-				return NULL;
-
-			/* sanity check flags */
-			if (g_strcmp0(keys[j], FU_QUIRKS_FLAGS) == 0) {
-				if (!fu_quirks_validate_flags(value, &error_local)) {
-					g_warning("[%s] %s = %s is invalid: %s",
-						  groups[i],
-						  keys[j],
-						  value,
-						  error_local->message);
-				}
-			}
-
-			xb_builder_node_insert_text(bn, "value", value, "key", keys[j], NULL);
-		}
-	}
-
 	/* export as XML blob */
-	xml = xb_builder_node_export(root, XB_NODE_EXPORT_FLAG_ADD_HEADER, error);
+	xml = xb_builder_node_export(helper->root, XB_NODE_EXPORT_FLAG_ADD_HEADER, error);
 	if (xml == NULL)
 		return NULL;
 	xmlsz = strlen(xml);
@@ -248,7 +292,7 @@ fu_quirks_add_quirks_for_path(FuQuirks *self, XbBuilder *builder, const gchar *p
 	if (dir == NULL)
 		return FALSE;
 	while ((tmp = g_dir_read_name(dir)) != NULL) {
-		if (!g_str_has_suffix(tmp, ".quirk")) {
+		if (!g_str_has_suffix(tmp, ".quirk") && !g_str_has_suffix(tmp, ".quirk.gz")) {
 			g_debug("skipping invalid file %s", tmp);
 			continue;
 		}
@@ -267,13 +311,13 @@ fu_quirks_add_quirks_for_path(FuQuirks *self, XbBuilder *builder, const gchar *p
 		/* load from keyfile */
 #if LIBXMLB_CHECK_VERSION(0, 1, 15)
 		xb_builder_source_add_simple_adapter(source,
-						     "text/plain,.quirk",
+						     "text/plain,application/octet-stream,.quirk",
 						     fu_quirks_convert_quirk_to_xml_cb,
 						     self,
 						     NULL);
 #else
 		xb_builder_source_add_adapter(source,
-					      "text/plain,.quirk",
+					      "text/plain,application/octet-stream,.quirk",
 					      fu_quirks_convert_quirk_to_xml_cb,
 					      self,
 					      NULL);
@@ -304,76 +348,6 @@ fu_quirks_strcasecmp_cb(gconstpointer a, gconstpointer b)
 	return g_ascii_strcasecmp(entry1, entry2);
 }
 
-static gboolean
-fu_quirks_add_quirks_for_resource(FuQuirks *self,
-				  XbBuilder *builder,
-				  const gchar *path,
-				  GError **error)
-{
-	g_autoptr(GBytes) blob = NULL;
-	g_autoptr(GBytes) blob_xml = NULL;
-	g_autoptr(XbBuilderSource) source = xb_builder_source_new();
-
-	/* load from keyfile */
-	blob = g_resources_lookup_data(path, G_RESOURCE_LOOKUP_FLAGS_NONE, error);
-	if (blob == NULL)
-		return FALSE;
-	blob_xml = fu_quirks_convert_keyfile_to_xml(self, blob, error);
-	if (blob_xml == NULL)
-		return FALSE;
-	if (!xb_builder_source_load_bytes(source, blob_xml, XB_BUILDER_SOURCE_FLAG_NONE, error)) {
-		g_prefix_error(error, "failed to load %s: ", path);
-		return FALSE;
-	}
-	xb_builder_import_source(builder, source);
-	return TRUE;
-}
-
-static gboolean
-fu_quirks_add_quirks_for_resources(FuQuirks *self,
-				   XbBuilder *builder,
-				   const gchar *path,
-				   GError **error)
-{
-	g_autofree gchar *hash_self = NULL;
-	g_auto(GStrv) children = NULL;
-	g_autoptr(GBytes) blob_self = NULL;
-	g_autoptr(GError) error_local = NULL;
-#ifdef _WIN32
-	gchar self_exe[MAX_PATH];
-	GetModuleFileNameA(NULL, self_exe, MAX_PATH);
-#else
-	const gchar *self_exe = "/proc/self/exe";
-#endif
-
-	children = g_resources_enumerate_children(path, G_RESOURCE_LOOKUP_FLAGS_NONE, &error_local);
-	if (children == NULL) {
-		if (g_error_matches(error_local, G_RESOURCE_ERROR, G_RESOURCE_ERROR_NOT_FOUND)) {
-			g_debug("ignoring: %s", error_local->message);
-			return TRUE;
-		}
-		g_propagate_error(error, g_steal_pointer(&error_local));
-		return FALSE;
-	}
-	for (guint i = 0; children[i] != NULL; i++) {
-		g_autofree gchar *fn = g_build_filename(path, children[i], NULL);
-		if (!g_str_has_suffix(children[i], ".quirk"))
-			continue;
-		if (!fu_quirks_add_quirks_for_resource(self, builder, fn, error))
-			return FALSE;
-	}
-
-	/* add the hash of the current binary to verify it has not changed */
-	blob_self = fu_bytes_get_contents(self_exe, error);
-	if (blob_self == NULL)
-		return FALSE;
-	hash_self = g_compute_checksum_for_bytes(G_CHECKSUM_SHA256, blob_self);
-	xb_builder_append_guid(builder, hash_self);
-
-	/* success */
-	return TRUE;
-}
-
 static gboolean
 fu_quirks_check_silo(FuQuirks *self, GError **error)
 {
@@ -388,12 +362,8 @@ fu_quirks_check_silo(FuQuirks *self, GError **error)
 	if (self->silo != NULL && xb_silo_is_valid(self->silo))
 		return TRUE;
 
-	/* built-in quirks */
-	builder = xb_builder_new();
-	if (!fu_quirks_add_quirks_for_resources(self, builder, "/org/freedesktop/fwupd", error))
-		return FALSE;
-
 	/* system datadir */
+	builder = xb_builder_new();
 	datadir = fu_path_from_kind(FU_PATH_KIND_DATADIR_QUIRKS);
 	if (!fu_quirks_add_quirks_for_path(self, builder, datadir, error))
 		return FALSE;
