@@ -27,22 +27,22 @@ static void
 fu_cpu_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuCpuDevice *self = FU_CPU_DEVICE(device);
-	fu_common_string_append_kb(str,
-				   idt,
-				   "HasSHSTK",
-				   fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_SHSTK));
-	fu_common_string_append_kb(str,
-				   idt,
-				   "HasIBT",
-				   fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_IBT));
-	fu_common_string_append_kb(str,
-				   idt,
-				   "HasTME",
-				   fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_TME));
-	fu_common_string_append_kb(str,
-				   idt,
-				   "HasSMAP",
-				   fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_SMAP));
+	fu_string_append_kb(str,
+			    idt,
+			    "HasSHSTK",
+			    fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_SHSTK));
+	fu_string_append_kb(str,
+			    idt,
+			    "HasIBT",
+			    fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_IBT));
+	fu_string_append_kb(str,
+			    idt,
+			    "HasTME",
+			    fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_TME));
+	fu_string_append_kb(str,
+			    idt,
+			    "HasSMAP",
+			    fu_cpu_device_has_flag(self, FU_CPU_DEVICE_FLAG_SMAP));
 }
 
 static const gchar *
@@ -104,7 +104,7 @@ fu_cpu_device_convert_vendor(const gchar *vendor)
 static void
 fu_cpu_device_init(FuCpuDevice *self)
 {
-	fu_device_add_guid_full(FU_DEVICE(self), "cpu", FU_DEVICE_INSTANCE_FLAG_NO_QUIRKS);
+	fu_device_add_instance_id_full(FU_DEVICE(self), "cpu", FU_DEVICE_INSTANCE_FLAG_VISIBLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_INTERNAL);
 	fu_device_add_icon(FU_DEVICE(self), "computer");
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_HEX);
@@ -121,12 +121,9 @@ fu_cpu_device_add_instance_ids(FuDevice *device, GError **error)
 	guint32 model_id_ext;
 	guint32 processor_id;
 	guint32 stepping_id;
-	g_autofree gchar *devid1 = NULL;
-	g_autofree gchar *devid2 = NULL;
-	g_autofree gchar *devid3 = NULL;
 
 	/* decode according to https://en.wikipedia.org/wiki/CPUID */
-	if (!fu_common_cpuid(0x1, &eax, NULL, NULL, NULL, error))
+	if (!fu_cpuid(0x1, &eax, NULL, NULL, NULL, error))
 		return FALSE;
 	processor_id = (eax >> 12) & 0x3;
 	model_id = (eax >> 4) & 0xf;
@@ -141,17 +138,16 @@ fu_cpu_device_add_instance_ids(FuDevice *device, GError **error)
 	if (family_id == 15)
 		family_id += family_id_ext;
 
-	devid1 = g_strdup_printf("CPUID\\PRO_%01X&FAM_%02X", processor_id, family_id);
-	fu_device_add_instance_id(device, devid1);
-	devid2 =
-	    g_strdup_printf("CPUID\\PRO_%01X&FAM_%02X&MOD_%02X", processor_id, family_id, model_id);
-	fu_device_add_instance_id(device, devid2);
-	devid3 = g_strdup_printf("CPUID\\PRO_%01X&FAM_%02X&MOD_%02X&STP_%01X",
-				 processor_id,
-				 family_id,
-				 model_id,
-				 stepping_id);
-	fu_device_add_instance_id(device, devid3);
+	/* add GUIDs */
+	fu_device_add_instance_u4(device, "PRO", processor_id);
+	fu_device_add_instance_u8(device, "FAM", family_id);
+	fu_device_add_instance_u8(device, "MOD", model_id);
+	fu_device_add_instance_u4(device, "STP", stepping_id);
+	fu_device_build_instance_id(device, NULL, "CPUID", "PRO", "FAM", NULL);
+	fu_device_build_instance_id(device, NULL, "CPUID", "PRO", "FAM", "MOD", NULL);
+	fu_device_build_instance_id(device, NULL, "CPUID", "PRO", "FAM", "MOD", "STP", NULL);
+
+	/* success */
 	return TRUE;
 }
 
@@ -162,7 +158,7 @@ fu_cpu_device_probe_manufacturer_id(FuDevice *device, GError **error)
 	guint32 ecx = 0;
 	guint32 edx = 0;
 	gchar str[13] = {'\0'};
-	if (!fu_common_cpuid(0x0, NULL, &ebx, &ecx, &edx, error))
+	if (!fu_cpuid(0x0, NULL, &ebx, &ecx, &edx, error))
 		return FALSE;
 	if (!fu_memcpy_safe((guint8 *)str,
 			    sizeof(str),
@@ -205,7 +201,7 @@ fu_cpu_device_probe_model(FuDevice *device, GError **error)
 	gchar str[49] = {'\0'};
 
 	for (guint32 i = 0; i < 3; i++) {
-		if (!fu_common_cpuid(0x80000002 + i, &eax, &ebx, &ecx, &edx, error))
+		if (!fu_cpuid(0x80000002 + i, &eax, &ebx, &ecx, &edx, error))
 			return FALSE;
 		if (!fu_memcpy_safe((guint8 *)str,
 				    sizeof(str),
@@ -254,8 +250,9 @@ fu_cpu_device_probe_extended_features(FuDevice *device, GError **error)
 	FuCpuDevice *self = FU_CPU_DEVICE(device);
 	guint32 ebx = 0;
 	guint32 ecx = 0;
+	guint32 edx = 0;
 
-	if (!fu_common_cpuid(0x7, NULL, &ebx, &ecx, NULL, error))
+	if (!fu_cpuid(0x7, NULL, &ebx, &ecx, &edx, error))
 		return FALSE;
 	if ((ebx >> 20) & 0x1)
 		self->flags |= FU_CPU_DEVICE_FLAG_SMAP;
@@ -263,7 +260,7 @@ fu_cpu_device_probe_extended_features(FuDevice *device, GError **error)
 		self->flags |= FU_CPU_DEVICE_FLAG_SHSTK;
 	if ((ecx >> 13) & 0x1)
 		self->flags |= FU_CPU_DEVICE_FLAG_TME;
-	if ((ecx >> 20) & 0x1)
+	if ((edx >> 20) & 0x1)
 		self->flags |= FU_CPU_DEVICE_FLAG_IBT;
 	return TRUE;
 }
@@ -287,7 +284,7 @@ fu_cpu_device_set_quirk_kv(FuDevice *device, const gchar *key, const gchar *valu
 {
 	if (g_strcmp0(key, "PciBcrAddr") == 0) {
 		guint64 tmp = 0;
-		if (!fu_common_strtoull_full(value, &tmp, 0, G_MAXUINT32, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT32, error))
 			return FALSE;
 		fu_device_set_metadata_integer(device, "PciBcrAddr", tmp);
 		return TRUE;
@@ -302,10 +299,8 @@ fu_cpu_device_add_security_attrs_intel_cet_enabled(FuCpuDevice *self, FuSecurity
 	g_autoptr(FwupdSecurityAttr) attr = NULL;
 
 	/* create attr */
-	attr = fwupd_security_attr_new(FWUPD_SECURITY_ATTR_ID_INTEL_CET_ENABLED);
-	fwupd_security_attr_set_plugin(attr, fu_device_get_plugin(FU_DEVICE(self)));
-	fwupd_security_attr_set_level(attr, FWUPD_SECURITY_ATTR_LEVEL_THEORETICAL);
-	fwupd_security_attr_add_guids(attr, fu_device_get_guids(FU_DEVICE(self)));
+	attr =
+	    fu_device_security_attr_new(FU_DEVICE(self), FWUPD_SECURITY_ATTR_ID_INTEL_CET_ENABLED);
 	fu_security_attrs_append(attrs, attr);
 
 	/* check for CET */
@@ -334,10 +329,8 @@ fu_cpu_device_add_security_attrs_intel_cet_active(FuCpuDevice *self, FuSecurityA
 		return;
 
 	/* create attr */
-	attr = fwupd_security_attr_new(FWUPD_SECURITY_ATTR_ID_INTEL_CET_ACTIVE);
-	fwupd_security_attr_set_plugin(attr, fu_device_get_plugin(FU_DEVICE(self)));
-	fwupd_security_attr_set_level(attr, FWUPD_SECURITY_ATTR_LEVEL_THEORETICAL);
-	fwupd_security_attr_add_guids(attr, fu_device_get_guids(FU_DEVICE(self)));
+	attr =
+	    fu_device_security_attr_new(FU_DEVICE(self), FWUPD_SECURITY_ATTR_ID_INTEL_CET_ACTIVE);
 	fwupd_security_attr_add_flag(attr, FWUPD_SECURITY_ATTR_FLAG_RUNTIME_ISSUE);
 	fu_security_attrs_append(attrs, attr);
 
@@ -367,10 +360,7 @@ fu_cpu_device_add_security_attrs_intel_tme(FuCpuDevice *self, FuSecurityAttrs *a
 	g_autoptr(FwupdSecurityAttr) attr = NULL;
 
 	/* create attr */
-	attr = fwupd_security_attr_new(FWUPD_SECURITY_ATTR_ID_ENCRYPTED_RAM);
-	fwupd_security_attr_set_plugin(attr, fu_device_get_plugin(FU_DEVICE(self)));
-	fwupd_security_attr_set_level(attr, FWUPD_SECURITY_ATTR_LEVEL_SYSTEM_PROTECTION);
-	fwupd_security_attr_add_guids(attr, fu_device_get_guids(FU_DEVICE(self)));
+	attr = fu_device_security_attr_new(FU_DEVICE(self), FWUPD_SECURITY_ATTR_ID_ENCRYPTED_RAM);
 	fu_security_attrs_append(attrs, attr);
 
 	/* check for TME */
@@ -390,10 +380,7 @@ fu_cpu_device_add_security_attrs_intel_smap(FuCpuDevice *self, FuSecurityAttrs *
 	g_autoptr(FwupdSecurityAttr) attr = NULL;
 
 	/* create attr */
-	attr = fwupd_security_attr_new(FWUPD_SECURITY_ATTR_ID_INTEL_SMAP);
-	fwupd_security_attr_set_plugin(attr, fu_device_get_plugin(FU_DEVICE(self)));
-	fwupd_security_attr_set_level(attr, FWUPD_SECURITY_ATTR_LEVEL_SYSTEM_PROTECTION);
-	fwupd_security_attr_add_guids(attr, fu_device_get_guids(FU_DEVICE(self)));
+	attr = fu_device_security_attr_new(FU_DEVICE(self), FWUPD_SECURITY_ATTR_ID_INTEL_SMAP);
 	fu_security_attrs_append(attrs, attr);
 
 	/* check for SMEP and SMAP */
@@ -407,19 +394,38 @@ fu_cpu_device_add_security_attrs_intel_smap(FuCpuDevice *self, FuSecurityAttrs *
 	fwupd_security_attr_set_result(attr, FWUPD_SECURITY_ATTR_RESULT_ENABLED);
 }
 
+static void
+fu_cpu_device_add_supported_cpu_attribute(FuCpuDevice *self, FuSecurityAttrs *attrs)
+{
+	g_autoptr(FwupdSecurityAttr) attr = NULL;
+
+	attr = fu_device_security_attr_new(FU_DEVICE(self), FWUPD_SECURITY_ATTR_ID_SUPPORTED_CPU);
+	switch (fu_cpu_get_vendor()) {
+	case FU_CPU_VENDOR_INTEL:
+	case FU_CPU_VENDOR_AMD:
+		fwupd_security_attr_add_flag(attr, FWUPD_SECURITY_ATTR_FLAG_SUCCESS);
+		fwupd_security_attr_set_result(attr, FWUPD_SECURITY_ATTR_RESULT_VALID);
+		break;
+	default:
+		fwupd_security_attr_add_flag(attr, FWUPD_SECURITY_ATTR_FLAG_MISSING_DATA);
+	}
+	fu_security_attrs_append(attrs, attr);
+}
+
 static void
 fu_cpu_device_add_security_attrs(FuDevice *device, FuSecurityAttrs *attrs)
 {
 	FuCpuDevice *self = FU_CPU_DEVICE(device);
 
 	/* only Intel */
-	if (fu_common_get_cpu_vendor() != FU_CPU_VENDOR_INTEL)
-		return;
+	if (fu_cpu_get_vendor() == FU_CPU_VENDOR_INTEL) {
+		fu_cpu_device_add_security_attrs_intel_cet_enabled(self, attrs);
+		fu_cpu_device_add_security_attrs_intel_cet_active(self, attrs);
+		fu_cpu_device_add_security_attrs_intel_tme(self, attrs);
+		fu_cpu_device_add_security_attrs_intel_smap(self, attrs);
+	}
 
-	fu_cpu_device_add_security_attrs_intel_cet_enabled(self, attrs);
-	fu_cpu_device_add_security_attrs_intel_cet_active(self, attrs);
-	fu_cpu_device_add_security_attrs_intel_tme(self, attrs);
-	fu_cpu_device_add_security_attrs_intel_smap(self, attrs);
+	fu_cpu_device_add_supported_cpu_attribute(self, attrs);
 }
 
 static void
