@@ -152,12 +152,12 @@ fu_dell_dock_tbt_set_quirk_kv(FuDevice *device,
 	guint64 tmp = 0;
 
 	if (g_strcmp0(key, "DellDockUnlockTarget") == 0) {
-		if (!fu_common_strtoull_full(value, &tmp, 0, G_MAXUINT8, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT8, error))
 			return FALSE;
 		self->unlock_target = tmp;
 		return TRUE;
 	} else if (g_strcmp0(key, "DellDockInstallDurationI2C") == 0) {
-		if (!fu_common_strtoull_full(value, &tmp, 0, 60 * 60 * 24, error))
+		if (!fu_strtoull(value, &tmp, 0, 60 * 60 * 24, error))
 			return FALSE;
 		fu_device_set_install_duration(device, tmp);
 		return TRUE;
@@ -165,12 +165,12 @@ fu_dell_dock_tbt_set_quirk_kv(FuDevice *device,
 		self->hub_minimum_version = g_strdup(value);
 		return TRUE;
 	} else if (g_strcmp0(key, "DellDockBlobMajorOffset") == 0) {
-		if (!fu_common_strtoull_full(value, &tmp, 0, G_MAXUINT32, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT32, error))
 			return FALSE;
 		self->blob_major_offset = tmp;
 		return TRUE;
 	} else if (g_strcmp0(key, "DellDockBlobMinorOffset") == 0) {
-		if (!fu_common_strtoull_full(value, &tmp, 0, G_MAXUINT32, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT32, error))
 			return FALSE;
 		self->blob_minor_offset = tmp;
 		return TRUE;
@@ -198,7 +198,7 @@ fu_dell_dock_tbt_setup(FuDevice *device, GError **error)
 
 	/* minimum version of NVM that supports this feature */
 	if (version == NULL ||
-	    fu_common_vercmp_full(version, MIN_NVM, FWUPD_VERSION_FORMAT_PAIR) < 0) {
+	    fu_version_compare(version, MIN_NVM, FWUPD_VERSION_FORMAT_PAIR) < 0) {
 		fu_device_set_update_error(
 		    device,
 		    "Updates over I2C are disabled due to insuffient NVM version");
@@ -206,9 +206,8 @@ fu_dell_dock_tbt_setup(FuDevice *device, GError **error)
 	}
 	/* minimum Hub2 version that supports this feature */
 	hub_version = fu_device_get_version(fu_device_get_proxy(device));
-	if (fu_common_vercmp_full(hub_version,
-				  self->hub_minimum_version,
-				  FWUPD_VERSION_FORMAT_PAIR) < 0) {
+	if (fu_version_compare(hub_version, self->hub_minimum_version, FWUPD_VERSION_FORMAT_PAIR) <
+	    0) {
 		fu_device_set_update_error(
 		    device,
 		    "Updates over I2C are disabled due to insufficient USB 3.1 G2 hub version");
@@ -294,7 +293,8 @@ fu_dell_dock_tbt_class_init(FuDellDockTbtClass *klass)
 FuDellDockTbt *
 fu_dell_dock_tbt_new(FuDevice *proxy)
 {
-	FuDellDockTbt *self = g_object_new(FU_TYPE_DELL_DOCK_TBT, NULL);
+	FuContext *ctx = fu_device_get_context(proxy);
+	FuDellDockTbt *self = g_object_new(FU_TYPE_DELL_DOCK_TBT, "context", ctx, NULL);
 	fu_device_set_proxy(FU_DEVICE(self), proxy);
 	return self;
 }
