@@ -201,10 +201,10 @@ fu_dfu_csr_device_upload(FuDevice *device, FuProgress *progress, GError **error)
 			return NULL;
 		chunk_sz = g_bytes_get_size(chunk);
 
-		/* get the total size using the DFU_CSR header */
+		/* get the total size using the CSR header */
 		if (i == 0 && chunk_sz >= 10) {
 			const guint8 *buf = g_bytes_get_data(chunk, NULL);
-			if (memcmp(buf, "DFU_CSR-dfu", 7) == 0) {
+			if (memcmp(buf, "CSR-dfu", 7) == 0) {
 				guint16 hdr_ver;
 				guint16 hdr_len;
 				if (!fu_memread_uint16_safe(buf,
@@ -288,15 +288,17 @@ fu_dfu_csr_device_download_chunk(FuDfuCsrDevice *self, guint16 idx, GBytes *chun
 	buf[1] = FU_DFU_CSR_COMMAND_UPGRADE;
 	fu_memwrite_uint16(&buf[2], idx, G_LITTLE_ENDIAN);
 	fu_memwrite_uint16(&buf[4], chunk_sz, G_LITTLE_ENDIAN);
-	if (!fu_memcpy_safe(buf,
-			    sizeof(buf),
-			    FU_DFU_CSR_COMMAND_HEADER_SIZE, /* dst */
-			    chunk_data,
-			    chunk_sz,
-			    0x0, /* src */
-			    chunk_sz,
-			    error))
-		return FALSE;
+	if (chunk_sz > 0) {
+		if (!fu_memcpy_safe(buf,
+				    sizeof(buf),
+				    FU_DFU_CSR_COMMAND_HEADER_SIZE, /* dst */
+				    chunk_data,
+				    chunk_sz,
+				    0x0, /* src */
+				    chunk_sz,
+				    error))
+			return FALSE;
+	}
 
 	/* hit hardware */
 	if (!fu_hid_device_set_report(FU_HID_DEVICE(self),
