@@ -16,7 +16,7 @@
 #include "fu-elantp-i2c-device.h"
 
 struct _FuElantpI2cDevice {
-	FuUdevDevice parent_instance;
+	FuI2cDevice parent_instance;
 	guint16 i2c_addr;
 	guint16 ic_page_count;
 	guint16 iap_type;
@@ -29,7 +29,7 @@ struct _FuElantpI2cDevice {
 	gchar *bind_id;
 };
 
-G_DEFINE_TYPE(FuElantpI2cDevice, fu_elantp_i2c_device, FU_TYPE_UDEV_DEVICE)
+G_DEFINE_TYPE(FuElantpI2cDevice, fu_elantp_i2c_device, FU_TYPE_I2C_DEVICE)
 
 #define FU_ELANTP_DEVICE_IOCTL_TIMEOUT 5000 /* ms */
 
@@ -807,6 +807,7 @@ fu_elantp_i2c_device_init(FuElantpI2cDevice *self)
 	fu_device_add_icon(FU_DEVICE(self), "input-touchpad");
 	fu_device_add_protocol(FU_DEVICE(self), "tw.com.emc.elantp");
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_HEX);
+	fu_device_set_vendor(FU_DEVICE(self), "ELAN Microelectronics");
 	fu_udev_device_set_flags(FU_UDEV_DEVICE(self),
 				 FU_UDEV_DEVICE_FLAG_OPEN_READ | FU_UDEV_DEVICE_FLAG_OPEN_WRITE);
 	fu_device_register_private_flag(FU_DEVICE(self),
