@@ -20,20 +20,9 @@ G_DEFINE_TYPE(FuHailuckTpDevice, fu_hailuck_tp_device, FU_TYPE_DEVICE)
 static gboolean
 fu_hailuck_tp_device_probe(FuDevice *device, GError **error)
 {
-	FuDevice *parent = fu_device_get_parent(device);
-	g_autofree gchar *devid1 = NULL;
-	g_autofree gchar *devid2 = NULL;
-
-	devid1 = g_strdup_printf("USB\\VID_%04X&PID_%04X",
-				 fu_usb_device_get_vid(FU_USB_DEVICE(parent)),
-				 fu_usb_device_get_pid(FU_USB_DEVICE(parent)));
-	fu_device_add_instance_id(device, devid1);
-	devid2 = g_strdup_printf("USB\\VID_%04X&PID_%04X&MODE_TP",
-				 fu_usb_device_get_vid(FU_USB_DEVICE(parent)),
-				 fu_usb_device_get_pid(FU_USB_DEVICE(parent)));
-	fu_device_add_instance_id(device, devid2);
-
-	return TRUE;
+	/* add extra touchpad-specific GUID */
+	fu_device_add_instance_str(device, "MODE", "TP");
+	return fu_device_build_instance_id(device, error, "USB", "VID", "PID", "MODE", NULL);
 }
 
 typedef struct {
@@ -105,11 +94,11 @@ fu_hailuck_tp_device_write_firmware(FuDevice *device,
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 10);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 85);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 1); /* end-program */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_VERIFY, 3);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 1); /* pass */
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 10, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 85, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 1, "end-program");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_VERIFY, 3, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 1, "pass");
 
 	/* get default image */
 	fw = fu_firmware_get_bytes(firmware, error);
@@ -215,10 +204,10 @@ fu_hailuck_tp_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2); /* detach */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 94);	/* write */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2); /* attach */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 2);	/* reload */
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, "detach");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 94, "write");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, "attach");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 2, "reload");
 }
 
 static void
