@@ -1509,6 +1509,7 @@ fu_mm_device_write_firmware_firehose(FuDevice *device,
 				     GError **error)
 {
 	FuMmDevice *self = FU_MM_DEVICE(device);
+	MMModem *modem = mm_object_peek_modem(self->omodem);
 	GBytes *firehose_rawprogram;
 	GBytes *firehose_prog;
 	g_autoptr(FuDeviceLocker) locker = NULL;
@@ -1550,10 +1551,11 @@ fu_mm_device_write_firmware_firehose(FuDevice *device,
 	/* Firehose program needs to be loaded to the modem before firehose update process can
 	 * start. Generally, modems use Sahara protocol to load the firehose binary.
 	 *
-	 * In case of MHI PCI modems, the mhi_wwan driver reads the firehose binary from the
+	 * In case of MHI PCI modems, the mhi-pci-generic driver reads the firehose binary from the
 	 * firmware-loader and writes it to the modem.
 	 **/
-	if (g_strrstr(fu_device_get_physical_id(device), "pci") && self->port_qcdm != NULL) {
+	if (g_strv_contains(mm_modem_get_drivers(modem), "mhi-pci-generic") &&
+	    self->port_qcdm != NULL) {
 		/* modify firmware search path and restore it before function returns */
 		locker = fu_device_locker_new_full(
 		    self,
@@ -1822,14 +1824,16 @@ fu_mm_device_setup_secboot_status_quectel(FuMmDevice *self)
 			return;
 		}
 
-		/* Try to query sec boot status with AT commands */
+		/* try to query sec boot status with AT commands */
 		for (guint i = 0; at_cmd[i] != NULL; i++) {
-			if (!fu_mm_device_at_cmd(self, at_cmd[i], TRUE, &error_local))
+			g_autoptr(GError) error_loop = NULL;
+			if (!fu_mm_device_at_cmd(self, at_cmd[i], TRUE, &error_loop)) {
 				g_debug("AT command failed (%s): %s",
 					at_cmd[i],
-					error_local->message);
-			else
+					error_loop->message);
+			} else {
 				return;
+			}
 		}
 	}
 
@@ -1911,6 +1915,7 @@ fu_mm_device_init(FuMmDevice *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_REPLUG_MATCH_GUID);
 	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_VERFMT);
+	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_PLAIN);
 	fu_device_set_summary(FU_DEVICE(self), "Mobile broadband device");
 	fu_device_add_icon(FU_DEVICE(self), "modem");
