@@ -59,8 +59,8 @@ fu_superio_it55_device_to_string(FuDevice *device, guint idt, GString *str)
 	/* FuSuperioDevice->to_string */
 	FU_DEVICE_CLASS(fu_superio_it55_device_parent_class)->to_string(device, idt, str);
 
-	fu_common_string_append_kv(str, idt, "PrjName", self->prj_name);
-	fu_common_string_append_kx(str, idt, "AutoloadAction", self->autoload_action);
+	fu_string_append(str, idt, "PrjName", self->prj_name);
+	fu_string_append_kx(str, idt, "AutoloadAction", self->autoload_action);
 }
 
 static gboolean
@@ -237,6 +237,8 @@ fu_superio_it55_device_get_firmware(FuDevice *device, FuProgress *progress, GErr
 	g_autofree guint8 *buf = NULL;
 
 	buf = g_malloc0(fwsize);
+	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_set_steps(progress, block_count);
 	for (guint i = 0; i < block_count; ++i) {
 		if (!fu_superio_device_ec_write_cmd(self, SIO_CMD_EC_READ_BLOCK, error) ||
 		    !fu_superio_device_ec_write_cmd(self, i, error))
@@ -245,9 +247,8 @@ fu_superio_it55_device_get_firmware(FuDevice *device, FuProgress *progress, GErr
 		for (guint j = 0; j < BLOCK_SIZE; ++j, ++offset) {
 			if (!fu_superio_device_ec_read_data(self, &buf[offset], error))
 				return NULL;
-
-			fu_progress_set_percentage_full(progress, (gsize)i + 1, (gsize)block_count);
 		}
+		fu_progress_step_done(progress);
 	}
 
 	return g_bytes_new_take(g_steal_pointer(&buf), fwsize);
@@ -341,10 +342,10 @@ fu_superio_it55_device_write_attempt(FuDevice *device,
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 10);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 80);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 1); /* block 0 */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_VERIFY, 9);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 10, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 80, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 1, "device-write-blk0");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_VERIFY, 9, NULL);
 
 	if (!fu_superio_it55_device_erase(device, error))
 		return FALSE;
@@ -352,10 +353,10 @@ fu_superio_it55_device_write_attempt(FuDevice *device,
 	erased_fw =
 	    fu_superio_it55_device_get_firmware(device, fu_progress_get_child(progress), error);
 	if (erased_fw == NULL) {
-		g_prefix_error(error, "failed to read erased firmware");
+		g_prefix_error(error, "failed to read erased firmware: ");
 		return FALSE;
 	}
-	if (!fu_common_bytes_is_empty(erased_fw)) {
+	if (!fu_bytes_is_empty(erased_fw)) {
 		g_set_error_literal(error,
 				    FWUPD_ERROR,
 				    FWUPD_ERROR_READ,
@@ -421,11 +422,11 @@ fu_superio_it55_device_write_attempt(FuDevice *device,
 	written_fw =
 	    fu_superio_it55_device_get_firmware(device, fu_progress_get_child(progress), error);
 	if (written_fw == NULL) {
-		g_prefix_error(error, "failed to read firmware");
+		g_prefix_error(error, "failed to read firmware: ");
 		return FALSE;
 	}
-	if (!fu_common_bytes_compare(written_fw, firmware, error)) {
-		g_prefix_error(error, "firmware verification");
+	if (!fu_bytes_compare(written_fw, firmware, error)) {
+		g_prefix_error(error, "firmware verification: ");
 		return FALSE;
 	}
 	fu_progress_step_done(progress);
@@ -448,8 +449,8 @@ fu_superio_it55_device_write_firmware(FuDevice *device,
 
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 10);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 90);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 10, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 90, NULL);
 
 	/* require detach -> attach */
 	locker = fu_device_locker_new_full(device,
@@ -466,7 +467,7 @@ fu_superio_it55_device_write_firmware(FuDevice *device,
 
 	fwsize = g_bytes_get_size(fw);
 	if (fwsize < 1024) {
-		g_prefix_error(error, "firmware is too small: %u", (guint)fwsize);
+		g_prefix_error(error, "firmware is too small %u: ", (guint)fwsize);
 		return FALSE;
 	}
 
@@ -619,7 +620,6 @@ fu_superio_it55_device_init(FuEcIt55Device *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_ONLY_OFFLINE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_NEEDS_REBOOT);
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	/* version string example: 1.07.02TR1 */
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_PLAIN);
 }
