@@ -42,6 +42,8 @@
 
 #define FLASH_SETTLE_TIME 5000000 /* us */
 
+#define CAYENNE_FIRMWARE_SIZE 0x50000 /* bytes */
+
 /**
  * FU_SYNAPTICS_MST_DEVICE_FLAG_IGNORE_BOARD_ID:
  *
@@ -860,7 +862,17 @@ fu_synaptics_mst_device_update_cayenne_firmware(FuSynapticsMstDevice *self,
 	guint32 offset = 0;
 	guint32 write_loops = 0;
 
-	payload_len = 0x50000;
+	/* sanity check */
+	if (payload_len < CAYENNE_FIRMWARE_SIZE) {
+		g_set_error(error,
+			    G_IO_ERROR,
+			    G_IO_ERROR_INVALID_DATA,
+			    "payload too small, expected >=0x%x",
+			    (guint)CAYENNE_FIRMWARE_SIZE);
+		return FALSE;
+	}
+
+	payload_len = CAYENNE_FIRMWARE_SIZE;
 	write_loops = (payload_len / BLOCK_UNIT);
 	data_to_write = payload_len;
 
@@ -1436,21 +1448,17 @@ fu_synaptics_mst_device_rescan(FuDevice *device, GError **error)
 	switch (self->family) {
 	case FU_SYNAPTICS_MST_FAMILY_TESLA:
 		fu_device_set_firmware_size_max(device, 0x10000);
-		fu_device_add_instance_id_full(device,
-					       "MST-tesla",
-					       FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
+		fu_device_add_instance_id_full(device, "MST-tesla", FU_DEVICE_INSTANCE_FLAG_QUIRKS);
 		break;
 	case FU_SYNAPTICS_MST_FAMILY_LEAF:
 		fu_device_set_firmware_size_max(device, 0x10000);
-		fu_device_add_instance_id_full(device,
-					       "MST-leaf",
-					       FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
+		fu_device_add_instance_id_full(device, "MST-leaf", FU_DEVICE_INSTANCE_FLAG_QUIRKS);
 		break;
 	case FU_SYNAPTICS_MST_FAMILY_PANAMERA:
 		fu_device_set_firmware_size_max(device, 0x80000);
 		fu_device_add_instance_id_full(device,
 					       "MST-panamera",
-					       FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
+					       FU_DEVICE_INSTANCE_FLAG_QUIRKS);
 		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_DUAL_IMAGE);
 		break;
 	default:
