@@ -121,23 +121,23 @@ fu_uefi_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuUefiDevice *self = FU_UEFI_DEVICE(device);
 	FuUefiDevicePrivate *priv = GET_PRIVATE(self);
-	fu_common_string_append_kv(str, idt, "Kind", fu_uefi_device_kind_to_string(priv->kind));
-	fu_common_string_append_kv(str, idt, "FwClass", priv->fw_class);
-	fu_common_string_append_kx(str, idt, "CapsuleFlags", priv->capsule_flags);
-	fu_common_string_append_kx(str, idt, "FwVersion", priv->fw_version);
-	fu_common_string_append_kx(str, idt, "FwVersionLowest", priv->fw_version_lowest);
-	fu_common_string_append_kv(str,
-				   idt,
-				   "LastAttemptStatus",
-				   fu_uefi_device_status_to_string(priv->last_attempt_status));
-	fu_common_string_append_kx(str, idt, "LastAttemptVersion", priv->last_attempt_version);
+	fu_string_append(str, idt, "Kind", fu_uefi_device_kind_to_string(priv->kind));
+	fu_string_append(str, idt, "FwClass", priv->fw_class);
+	fu_string_append_kx(str, idt, "CapsuleFlags", priv->capsule_flags);
+	fu_string_append_kx(str, idt, "FwVersion", priv->fw_version);
+	fu_string_append_kx(str, idt, "FwVersionLowest", priv->fw_version_lowest);
+	fu_string_append(str,
+			 idt,
+			 "LastAttemptStatus",
+			 fu_uefi_device_status_to_string(priv->last_attempt_status));
+	fu_string_append_kx(str, idt, "LastAttemptVersion", priv->last_attempt_version);
 	if (priv->esp != NULL) {
-		fu_common_string_append_kv(str, idt, "EspId", fu_volume_get_id(priv->esp));
+		fu_string_append(str, idt, "EspId", fu_volume_get_id(priv->esp));
 	}
-	fu_common_string_append_ku(str,
-				   idt,
-				   "RequireESPFreeSpace",
-				   fu_device_get_metadata_integer(device, "RequireESPFreeSpace"));
+	fu_string_append_ku(str,
+			    idt,
+			    "RequireESPFreeSpace",
+			    fu_device_get_metadata_integer(device, "RequireESPFreeSpace"));
 }
 
 static void
@@ -383,7 +383,7 @@ fu_uefi_device_build_dp_buf(const gchar *path, gsize *bufsz, GError **error)
 	/* parse what we got back from efivar */
 	dps = fu_uefi_devpath_parse(dp_buf, (gsize)sz, FU_UEFI_DEVPATH_PARSE_FLAG_NONE, error);
 	if (dps == NULL) {
-		fu_common_dump_raw(G_LOG_DOMAIN, "dp_buf", dp_buf, (gsize)sz);
+		fu_dump_raw(G_LOG_DOMAIN, "dp_buf", dp_buf, (gsize)sz);
 		return NULL;
 	}
 
@@ -397,21 +397,21 @@ GBytes *
 fu_uefi_device_fixup_firmware(FuUefiDevice *self, GBytes *fw, GError **error)
 {
 	FuUefiDevicePrivate *priv = GET_PRIVATE(self);
-	gsize fw_length;
-	const guint8 *data = g_bytes_get_data(fw, &fw_length);
+	gsize bufsz;
+	const guint8 *buf = g_bytes_get_data(fw, &bufsz);
 	g_autofree gchar *guid_new = NULL;
 
 	priv->missing_header = FALSE;
 
 	/* GUID is the first 16 bytes */
-	if (fw_length < sizeof(fwupd_guid_t)) {
+	if (bufsz < sizeof(fwupd_guid_t)) {
 		g_set_error_literal(error,
 				    FWUPD_ERROR,
 				    FWUPD_ERROR_INVALID_FILE,
 				    "Invalid payload");
 		return NULL;
 	}
-	guid_new = fwupd_guid_to_string((fwupd_guid_t *)data, FWUPD_GUID_FLAG_MIXED_ENDIAN);
+	guid_new = fwupd_guid_to_string((fwupd_guid_t *)buf, FWUPD_GUID_FLAG_MIXED_ENDIAN);
 
 	/* ESRT header matches payload */
 	if (g_strcmp0(fu_uefi_device_get_guid(self), guid_new) == 0) {
@@ -422,17 +422,18 @@ fu_uefi_device_fixup_firmware(FuUefiDevice *self, GBytes *fw, GError **error)
 					      FU_UEFI_DEVICE_FLAG_NO_CAPSULE_HEADER_FIXUP)) {
 		return g_bytes_ref(fw);
 	} else {
-		guint header_size = getpagesize();
-		guint8 *new_data = g_malloc(fw_length + header_size);
-		guint8 *capsule = new_data + header_size;
+		guint hdrsize = getpagesize();
 		fwupd_guid_t esrt_guid = {0x0};
-		efi_capsule_header_t *header = (efi_capsule_header_t *)new_data;
+		efi_capsule_header_t header = {0x0};
+		g_autoptr(GByteArray) buf_hdr = g_byte_array_new();
 
 		g_warning("missing or invalid embedded capsule header");
 		priv->missing_header = TRUE;
-		header->flags = priv->capsule_flags;
-		header->header_size = header_size;
-		header->capsule_image_size = fw_length + header_size;
+
+		/* create a fake header with plausible contents */
+		header.flags = priv->capsule_flags;
+		header.header_size = hdrsize;
+		header.capsule_image_size = bufsz + hdrsize;
 		if (!fwupd_guid_from_string(fu_uefi_device_get_guid(self),
 					    &esrt_guid,
 					    FWUPD_GUID_FLAG_MIXED_ENDIAN,
@@ -440,10 +441,13 @@ fu_uefi_device_fixup_firmware(FuUefiDevice *self, GBytes *fw, GError **error)
 			g_prefix_error(error, "Invalid ESRT GUID: ");
 			return NULL;
 		}
-		memcpy(&header->guid, &esrt_guid, sizeof(fwupd_guid_t));
-		memcpy(capsule, data, fw_length);
+		memcpy(&header.guid, &esrt_guid, sizeof(fwupd_guid_t));
 
-		return g_bytes_new_take(new_data, fw_length + header_size);
+		/* prepend the header to the payload */
+		g_byte_array_append(buf_hdr, (const guint8 *)&header, sizeof(header));
+		fu_byte_array_set_size(buf_hdr, hdrsize, 0x00);
+		g_byte_array_append(buf_hdr, buf, bufsz);
+		return g_byte_array_free_to_bytes(g_steal_pointer(&buf_hdr));
 	}
 }
 
@@ -455,9 +459,8 @@ fu_uefi_device_write_update_info(FuUefiDevice *self,
 				 GError **error)
 {
 	FuUefiDevicePrivate *priv = GET_PRIVATE(self);
-	gsize datasz = 0;
 	gsize dp_bufsz = 0;
-	g_autofree guint8 *data = NULL;
+	g_autoptr(GByteArray) buf = g_byte_array_new();
 	g_autofree guint8 *dp_buf = NULL;
 	efi_update_info_t info = {
 	    .update_info_version = 0x7,
@@ -482,14 +485,12 @@ fu_uefi_device_write_update_info(FuUefiDevice *self,
 	/* save this header and body to the hardware */
 	if (!fwupd_guid_from_string(guid, &info.guid, FWUPD_GUID_FLAG_MIXED_ENDIAN, error))
 		return FALSE;
-	datasz = sizeof(info) + dp_bufsz;
-	data = g_malloc0(datasz);
-	memcpy(data, &info, sizeof(info));
-	memcpy(data + sizeof(info), dp_buf, dp_bufsz);
+	g_byte_array_append(buf, (const guint8 *)&info, sizeof(info));
+	g_byte_array_append(buf, dp_buf, dp_bufsz);
 	return fu_efivar_set_data(FU_EFIVAR_GUID_FWUPDATE,
 				  varname,
-				  data,
-				  datasz,
+				  buf->data,
+				  buf->len,
 				  FU_EFIVAR_ATTR_NON_VOLATILE | FU_EFIVAR_ATTR_BOOTSERVICE_ACCESS |
 				      FU_EFIVAR_ATTR_RUNTIME_ACCESS,
 				  error);
@@ -513,7 +514,7 @@ fu_uefi_check_asset(FuDevice *device, GError **error)
 {
 	g_autofree gchar *source_app = fu_uefi_get_built_app_path(error);
 	if (source_app == NULL) {
-		if (fu_efivar_secure_boot_enabled())
+		if (fu_efivar_secure_boot_enabled(NULL))
 			g_prefix_error(error, "missing signed bootloader for secure boot: ");
 		return FALSE;
 	}
@@ -535,13 +536,13 @@ fu_uefi_device_cleanup_esp(FuDevice *device, GError **error)
 		return TRUE;
 
 	/* delete any files matching the glob in the ESP */
-	files = fu_common_get_files_recursive(esp_path, error);
+	files = fu_path_get_files(esp_path, error);
 	if (files == NULL)
 		return FALSE;
 	pattern = g_build_filename(esp_path, "EFI/*/fw/fwupd*.cap", NULL);
 	for (guint i = 0; i < files->len; i++) {
 		const gchar *fn = g_ptr_array_index(files, i);
-		if (fu_common_fnmatch(pattern, fn)) {
+		if (fu_path_fnmatch(pattern, fn)) {
 			g_autoptr(GFile) file = g_file_new_for_path(fn);
 			g_debug("deleting %s", fn);
 			if (!g_file_delete(file, NULL, error))
@@ -557,7 +558,10 @@ fu_uefi_device_cleanup_esp(FuDevice *device, GError **error)
 }
 
 static gboolean
-fu_uefi_device_prepare(FuDevice *device, FwupdInstallFlags flags, GError **error)
+fu_uefi_device_prepare(FuDevice *device,
+		       FuProgress *progress,
+		       FwupdInstallFlags flags,
+		       GError **error)
 {
 	FuUefiDevice *self = FU_UEFI_DEVICE(device);
 	FuUefiDevicePrivate *priv = GET_PRIVATE(self);
@@ -579,7 +583,10 @@ fu_uefi_device_prepare(FuDevice *device, FwupdInstallFlags flags, GError **error
 }
 
 static gboolean
-fu_uefi_device_cleanup(FuDevice *device, FwupdInstallFlags flags, GError **error)
+fu_uefi_device_cleanup(FuDevice *device,
+		       FuProgress *progress,
+		       FwupdInstallFlags flags,
+		       GError **error)
 {
 	FuUefiDevice *self = FU_UEFI_DEVICE(device);
 	FuUefiDevicePrivate *priv = GET_PRIVATE(self);
@@ -625,13 +632,12 @@ fu_uefi_device_probe(FuDevice *device, GError **error)
 
 	/* set versions */
 	version_format = fu_device_get_version_format(device);
-	version = fu_common_version_from_uint32(priv->fw_version, version_format);
+	version = fu_version_from_uint32(priv->fw_version, version_format);
 	fu_device_set_version_format(device, version_format);
 	fu_device_set_version_raw(device, priv->fw_version);
 	fu_device_set_version(device, version);
 	if (priv->fw_version_lowest != 0) {
-		version_lowest =
-		    fu_common_version_from_uint32(priv->fw_version_lowest, version_format);
+		version_lowest = fu_version_from_uint32(priv->fw_version_lowest, version_format);
 		fu_device_set_version_lowest_raw(device, priv->fw_version_lowest);
 		fu_device_set_version_lowest(device, version_lowest);
 	}
@@ -645,11 +651,7 @@ fu_uefi_device_probe(FuDevice *device, GError **error)
 	fu_device_add_internal_flag(device, FU_DEVICE_INTERNAL_FLAG_MD_SET_VENDOR);
 
 	/* add icons */
-	if (priv->kind == FU_UEFI_DEVICE_KIND_DEVICE_FIRMWARE) {
-		/* nothing better in the icon naming spec */
-		fu_device_add_icon(device, "audio-card");
-	} else {
-		/* this is probably system firmware */
+	if (priv->kind == FU_UEFI_DEVICE_KIND_SYSTEM_FIRMWARE) {
 		fu_device_add_icon(device, "computer");
 		fu_device_add_instance_id(device, "main-system-firmware");
 	}
@@ -721,10 +723,10 @@ static void
 fu_uefi_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0); /* detach */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 100); /* write */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0); /* attach */
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 0);	/* reload */
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 100, "write");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "attach");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 0, "reload");
 }
 
 static void
