@@ -22,6 +22,7 @@ G_DEFINE_TYPE(FuUsiDockMcuDevice, fu_usi_dock_mcu_device, FU_TYPE_HID_DEVICE)
 
 #define FU_USI_DOCK_DEVICE_FLAG_VERFMT_HP (1 << 0)
 
+#define USI_DOCK_NON_IOT_INSTANCE_ID "USB\\VID_17EF&PID_30B4&CID_40B0"
 static gboolean
 fu_usi_dock_mcu_device_tx(FuUsiDockMcuDevice *self,
 			  guint8 tag2,
@@ -486,7 +487,7 @@ fu_usi_dock_mcu_device_write_chunks(FuUsiDockMcuDevice *self,
 	for (guint i = 0; i < chunks->len; i++) {
 		FuChunk *chk = g_ptr_array_index(chunks, i);
 		if (!fu_usi_dock_mcu_device_write_chunk(self, chk, error)) {
-			g_prefix_error(error, "failed to write chunk 0x%x", i);
+			g_prefix_error(error, "failed to write chunk 0x%x: ", i);
 			return FALSE;
 		}
 		fu_progress_step_done(progress);
@@ -523,6 +524,37 @@ fu_usi_dock_mcu_device_wait_for_spi_ready_cb(FuDevice *device, gpointer user_dat
 	return TRUE;
 }
 
+static gboolean
+fu_usi_dock_mcu_device_wait_for_spi_initial_ready_cb(FuDevice *device,
+						     gpointer user_data,
+						     GError **error)
+{
+	FuUsiDockMcuDevice *self = FU_USI_DOCK_MCU_DEVICE(device);
+	guint8 buf[] = {USBUID_ISP_DEVICE_CMD_FWBUFER_INITIAL};
+	guint8 val = 0;
+
+	if (!fu_usi_dock_mcu_device_txrx(self,
+					 TAG_TAG2_CMD_SPI,
+					 buf,
+					 sizeof(buf),
+					 &val,
+					 sizeof(val),
+					 error))
+		return FALSE;
+	if (val != SPI_STATE_READY) {
+		g_set_error(error,
+			    G_IO_ERROR,
+			    G_IO_ERROR_BUSY,
+			    "SPI state is %s [0x%02x]",
+			    fu_usi_dock_spi_state_to_string(val),
+			    val);
+		return FALSE;
+	}
+
+	/* success */
+	return TRUE;
+}
+
 static gboolean
 fu_usi_dock_mcu_device_wait_for_checksum_cb(FuDevice *device, gpointer user_data, GError **error)
 {
@@ -548,7 +580,6 @@ fu_usi_dock_mcu_device_write_firmware_with_idx(FuUsiDockMcuDevice *self,
 					       GError **error)
 {
 	guint8 cmd;
-	guint8 val = 0x0;
 	g_autoptr(GBytes) fw = NULL;
 	g_autoptr(GPtrArray) chunks = NULL;
 	guint8 checksum = 0xFF;
@@ -557,27 +588,18 @@ fu_usi_dock_mcu_device_write_firmware_with_idx(FuUsiDockMcuDevice *self,
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 0, NULL);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 6, NULL);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 40, "write-external");
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 13, "wait-for-checksum");
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 42, "internal-flash");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 5, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 69, "write-external");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 25, "wait-for-checksum");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 0, "internal-flash");
 
 	/* initial external flash */
-	cmd = USBUID_ISP_DEVICE_CMD_FWBUFER_INITIAL;
-	if (!fu_usi_dock_mcu_device_txrx(self,
-					 TAG_TAG2_CMD_SPI,
-					 &cmd,
-					 sizeof(cmd),
-					 &val,
-					 sizeof(val),
-					 error))
-		return FALSE;
-	if (val != SPI_STATE_READY) {
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
-			    "invalid state for CMD_FWBUFER_INITIAL, got 0x%02x",
-			    val);
+	if (!fu_device_retry(FU_DEVICE(self),
+			     fu_usi_dock_mcu_device_wait_for_spi_initial_ready_cb,
+			     30,
+			     NULL,
+			     error)) {
+		g_prefix_error(error, "failed to wait for initial: ");
 		return FALSE;
 	}
 	fu_progress_step_done(progress);
@@ -686,6 +708,30 @@ fu_usi_dock_mcu_device_write_firmware(FuDevice *device,
 							      error);
 }
 
+static gboolean
+fu_usi_dock_mcu_device_prepare(FuDevice *device,
+			       FuProgress *progress,
+			       FwupdInstallFlags flags,
+			       GError **error)
+{
+	FuUsiDockMcuDevice *self = FU_USI_DOCK_MCU_DEVICE(device);
+	guint8 inbuf[] = {USBUID_ISP_DEVICE_CMD_SET_CHIP_TYPE, 1, 1};
+
+	if (fu_device_has_guid(device, USI_DOCK_NON_IOT_INSTANCE_ID) &&
+	    g_strcmp0(fu_device_get_version(device), "10.10") == 0) {
+		if (!fu_usi_dock_mcu_device_txrx(self,
+						 TAG_TAG2_CMD_MCU,
+						 inbuf,
+						 sizeof(inbuf),
+						 NULL,
+						 0x0,
+						 error))
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 fu_usi_dock_mcu_device_attach(FuDevice *device, FuProgress *progress, GError **error)
 {
@@ -739,4 +785,5 @@ fu_usi_dock_mcu_device_class_init(FuUsiDockMcuDeviceClass *klass)
 	klass_device->attach = fu_usi_dock_mcu_device_attach;
 	klass_device->setup = fu_usi_dock_mcu_device_setup;
 	klass_device->set_progress = fu_usi_dock_mcu_device_set_progress;
+	klass_device->prepare = fu_usi_dock_mcu_device_prepare;
 }
