@@ -79,7 +79,7 @@ fu_wac_module_bluetooth_parse_blocks(const guint8 *data,
 	const guint8 preamble[] = {0x02, 0x00, 0x0f, 0x06, 0x01, 0x08, 0x01};
 	GPtrArray *blocks = g_ptr_array_new_with_free_func(g_free);
 	for (guint addr = 0x0; addr < sz; addr += FU_WAC_MODULE_BLUETOOTH_PAYLOAD_SZ) {
-		FuWacModuleBluetoothBlockData *bd;
+		g_autofree FuWacModuleBluetoothBlockData *bd = NULL;
 		gsize cdata_sz = FU_WAC_MODULE_BLUETOOTH_PAYLOAD_SZ;
 
 		/* user data area */
@@ -108,7 +108,7 @@ fu_wac_module_bluetooth_parse_blocks(const guint8 *data,
 			return NULL;
 		bd->crc = fu_wac_module_bluetooth_calculate_crc(bd->cdata,
 								FU_WAC_MODULE_BLUETOOTH_PAYLOAD_SZ);
-		g_ptr_array_add(blocks, bd);
+		g_ptr_array_add(blocks, g_steal_pointer(&bd));
 	}
 	return blocks;
 }
@@ -130,9 +130,9 @@ fu_wac_module_bluetooth_write_firmware(FuDevice *device,
 
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 20);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 79);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 1);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_ERASE, 20, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 79, NULL);
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 1, NULL);
 
 	/* get default image */
 	fw = fu_firmware_get_bytes(firmware, error);
