@@ -28,10 +28,12 @@ struct _FuRelease {
 	FwupdRemote *remote;
 	FuConfig *config;
 	GBytes *blob_fw;
+	gchar *update_request_id;
 	FwupdReleaseFlags trust_flags;
 	gboolean is_downgrade;
 	GPtrArray *soft_reqs; /* nullable, element-type XbNode */
 	GPtrArray *hard_reqs; /* nullable, element-type XbNode */
+	guint64 priority;
 };
 
 G_DEFINE_TYPE(FuRelease, fu_release, FWUPD_TYPE_RELEASE)
@@ -139,6 +141,34 @@ fu_release_get_hard_reqs(FuRelease *self)
 	return self->hard_reqs;
 }
 
+/**
+ * fu_release_get_update_request_id:
+ * @self: a #FuRelease
+ *
+ * Gets the update request ID as specified from `LVFS::UpdateRequestId`.
+ *
+ * Returns: a string value, or %NULL if never set.
+ **/
+const gchar *
+fu_release_get_update_request_id(FuRelease *self)
+{
+	g_return_val_if_fail(FU_IS_RELEASE(self), NULL);
+	return self->update_request_id;
+}
+
+static void
+fu_release_set_update_request_id(FuRelease *self, const gchar *update_request_id)
+{
+	g_return_if_fail(FU_IS_RELEASE(self));
+
+	/* not changed */
+	if (g_strcmp0(self->update_request_id, update_request_id) == 0)
+		return;
+
+	g_free(self->update_request_id);
+	self->update_request_id = g_strdup(update_request_id);
+}
+
 /**
  * fu_release_set_remote:
  * @self: a #FuRelease
@@ -215,6 +245,60 @@ fu_release_get_release_version(FuRelease *self, const gchar *version, GError **e
 	return fu_version_from_uint32((guint32)ver_uint32, fmt);
 }
 
+static gboolean
+fu_release_load_test_result(FuRelease *self, XbNode *n, GError **error)
+{
+	XbNode *c;
+	const gchar *tmp;
+	g_autoptr(FwupdReport) report = fwupd_report_new();
+	g_autoptr(GPtrArray) custom = NULL;
+
+	tmp = xb_node_get_attr(n, "date");
+	if (tmp != NULL) {
+		g_autoptr(GDateTime) dt = NULL;
+		g_autofree gchar *iso8601 = g_strdup_printf("%sT00:00:00Z", tmp);
+		dt = g_date_time_new_from_iso8601(iso8601, NULL);
+		if (dt != NULL)
+			fwupd_report_set_created(report, g_date_time_to_unix(dt));
+	}
+	tmp = xb_node_query_text(n, "device", NULL);
+	if (tmp != NULL)
+		fwupd_report_set_device_name(report, tmp);
+	tmp = xb_node_query_text(n, "previous_version", NULL);
+	if (tmp != NULL)
+		fwupd_report_set_version_old(report, tmp);
+	c = xb_node_query_first(n, "vendor_name", NULL);
+	if (c != NULL) {
+		guint64 vendor_id = xb_node_get_attr_as_uint(c, "id");
+		fwupd_report_set_vendor(report, xb_node_get_text(c));
+		if (vendor_id != G_MAXUINT64)
+			fwupd_report_set_vendor_id(report, vendor_id);
+	}
+	c = xb_node_query_first(n, "os", NULL);
+	if (c != NULL) {
+		tmp = xb_node_get_attr(c, "version");
+		if (tmp != NULL)
+			fwupd_report_set_distro_version(report, tmp);
+		tmp = xb_node_get_attr(c, "variant");
+		if (tmp != NULL)
+			fwupd_report_set_distro_variant(report, tmp);
+		fwupd_report_set_distro_id(report, xb_node_get_text(c));
+	}
+	custom = xb_node_query(n, "custom/value", 0, NULL);
+	if (custom != NULL) {
+		for (guint i = 0; i < custom->len; i++) {
+			c = g_ptr_array_index(custom, i);
+			fwupd_report_add_metadata_item(report,
+						       xb_node_get_attr(c, "key"),
+						       xb_node_get_text(c));
+		}
+	}
+
+	/* success */
+	fwupd_release_add_report(FWUPD_RELEASE(self), report);
+	return TRUE;
+}
+
 static gboolean
 fu_release_load_artifact(FuRelease *self, XbNode *artifact, GError **error)
 {
@@ -222,10 +306,11 @@ fu_release_load_artifact(FuRelease *self, XbNode *artifact, GError **error)
 	guint64 size;
 	g_autoptr(GPtrArray) locations = NULL;
 	g_autoptr(GPtrArray) checksums = NULL;
+	g_autoptr(GPtrArray) test_result = NULL;
 
 	/* filename */
 	filename = xb_node_query_text(artifact, "filename", NULL);
-	if (!g_str_has_suffix(filename, ".cab")) {
+	if (filename != NULL && !g_str_has_suffix(filename, ".cab")) {
 		/* some firmware archives was signed with <artifact type="binary"> where the
 		 * checksums were the *content* checksums, not the *container* checksum */
 		g_debug("ignoring non-binary artifact entry: %s", filename);
@@ -273,6 +358,16 @@ fu_release_load_artifact(FuRelease *self, XbNode *artifact, GError **error)
 		}
 	}
 
+	/* test results */
+	test_result = xb_node_query(artifact, "testing/test_result", 0, NULL);
+	if (test_result != NULL) {
+		for (guint i = 0; i < test_result->len; i++) {
+			XbNode *n = g_ptr_array_index(test_result, i);
+			if (!fu_release_load_test_result(self, n, error))
+				return FALSE;
+		}
+	}
+
 	/* size */
 	size = xb_node_query_text_as_uint(artifact, "size[@type='installed']", NULL);
 	if (size != G_MAXUINT64)
@@ -589,6 +684,13 @@ fu_release_check_requirements(FuRelease *self,
 	return TRUE;
 }
 
+void
+fu_release_set_priority(FuRelease *self, guint64 priority)
+{
+	g_return_if_fail(FU_IS_RELEASE(self));
+	self->priority = priority;
+}
+
 /**
  * fu_release_load:
  * @self: a #FuRelease
@@ -660,6 +762,9 @@ fu_release_load(FuRelease *self,
 	tmp = xb_node_query_text(component, "developer_name", NULL);
 	if (tmp != NULL)
 		fwupd_release_set_vendor(FWUPD_RELEASE(self), tmp);
+	tmp64 = xb_node_get_attr_as_uint(component, "priority");
+	if (tmp64 != G_MAXUINT64)
+		fu_release_set_priority(self, tmp64);
 
 	/* use default release */
 	if (rel_optional == NULL) {
@@ -873,6 +978,9 @@ fu_release_load(FuRelease *self,
 			fwupd_release_set_update_image(FWUPD_RELEASE(self), tmp);
 		}
 	}
+	tmp = xb_node_query_text(component, "custom/value[@key='LVFS::UpdateRequestId']", NULL);
+	if (tmp != NULL)
+		fu_release_set_update_request_id(self, tmp);
 
 	/* hard and soft requirements */
 	self->hard_reqs = xb_node_query(component, "requires/*", 0, &error_hard);
@@ -982,7 +1090,7 @@ fu_release_get_action_id(FuRelease *self)
  * @release1: first task to compare.
  * @release2: second task to compare.
  *
- * Compares two install tasks.
+ * Compares two releases.
  *
  * Returns: 1, 0 or -1 if @release1 is greater, equal, or less than @release2, respectively.
  **/
@@ -991,10 +1099,18 @@ fu_release_compare(FuRelease *release1, FuRelease *release2)
 {
 	FuDevice *device1 = fu_release_get_device(release1);
 	FuDevice *device2 = fu_release_get_device(release2);
+
+	/* device order, lower is better */
 	if (fu_device_get_order(device1) < fu_device_get_order(device2))
 		return -1;
 	if (fu_device_get_order(device1) > fu_device_get_order(device2))
 		return 1;
+
+	/* release priority, higher is better */
+	if (release1->priority > release2->priority)
+		return -1;
+	if (release1->priority < release2->priority)
+		return 1;
 	return 0;
 }
 
@@ -1009,6 +1125,7 @@ fu_release_finalize(GObject *obj)
 {
 	FuRelease *self = FU_RELEASE(obj);
 
+	g_free(self->update_request_id);
 	if (self->request != NULL)
 		g_object_unref(self->request);
 	if (self->device != NULL)
