@@ -217,7 +217,7 @@ fu_util_traverse_tree(GNode *n, gpointer data)
 		}
 
 		/* ancestors */
-		for (GNode *c = n->parent; c->parent != NULL; c = c->parent) {
+		for (GNode *c = n->parent; c != NULL && c->parent != NULL; c = c->parent) {
 			if (g_node_next_sibling(c) != NULL || idx == 0) {
 				g_string_prepend(str, "│ ");
 				continue;
@@ -264,6 +264,8 @@ fu_util_is_interesting_device(FwupdDevice *dev)
 		return TRUE;
 	if (fwupd_device_get_update_error(dev) != NULL)
 		return TRUE;
+	if (fwupd_device_get_version(dev) != NULL)
+		return TRUE;
 	/* device not plugged in, get-details */
 	if (fwupd_device_get_flags(dev) == 0)
 		return TRUE;
@@ -815,6 +817,11 @@ fu_util_release_get_name(FwupdRelease *release)
 			 * SATA or NVMe disk */
 			return g_strdup_printf(_("%s SSD Update"), name);
 		}
+		if (g_strcmp0(cat, "X-Gpu") == 0) {
+			/* TRANSLATORS: GPU refers to a Graphics Processing Unit, e.g.
+			 * the "video card" */
+			return g_strdup_printf(_("%s GPU Update"), name);
+		}
 	}
 
 	/* TRANSLATORS: this is the fallback where we don't know if the release
@@ -1349,7 +1356,7 @@ fu_util_device_problem_to_string(FwupdClient *client, FwupdDevice *dev, FwupdDev
 		if (fwupd_device_get_battery_level(dev) == FWUPD_BATTERY_LEVEL_INVALID ||
 		    fwupd_device_get_battery_threshold(dev) == FWUPD_BATTERY_LEVEL_INVALID) {
 			/* TRANSLATORS: for example the batteries *inside* the Bluetooth mouse */
-			return g_strdup_printf(_("Device battery power is too low"));
+			return g_strdup(_("Device battery power is too low"));
 		}
 		/* TRANSLATORS: for example the batteries *inside* the Bluetooth mouse */
 		return g_strdup_printf(_("Device battery power is too low (%u%%, requires %u%%)"),
@@ -1372,6 +1379,10 @@ fu_util_device_problem_to_string(FwupdClient *client, FwupdDevice *dev, FwupdDev
 		/* TRANSLATORS: emulated means we are pretending to be a different model */
 		return g_strdup(_("Device is emulated"));
 	}
+	if (problem == FWUPD_DEVICE_PROBLEM_MISSING_LICENSE) {
+		/* TRANSLATORS: The device cannot be updated due to missing vendor's license." */
+		return g_strdup(_("Device requires a software license to update"));
+	}
 	return NULL;
 }
 
@@ -1688,6 +1699,18 @@ fu_util_plugin_flag_to_string(FwupdPluginFlags plugin_flag)
 		/* TRANSLATORS: user needs to run a command */
 		return _("Authentication details are required");
 	}
+	if (plugin_flag == FWUPD_PLUGIN_FLAG_SECURE_CONFIG) {
+		/* TRANSLATORS: no peeking */
+		return _("Configuration is only readable by the system administrator");
+	}
+	if (plugin_flag == FWUPD_PLUGIN_FLAG_MODULAR) {
+		/* TRANSLATORS: the plugin was created from a .so object, and was not built-in */
+		return _("Loaded from an external module");
+	}
+	if (plugin_flag == FWUPD_PLUGIN_FLAG_MEASURE_SYSTEM_INTEGRITY) {
+		/* TRANSLATORS: check various UEFI and ACPI tables are unchanged after the update */
+		return _("Will measure elements of system integrity around an update");
+	}
 	if (plugin_flag == FWUPD_PLUGIN_FLAG_EFIVAR_NOT_MOUNTED) {
 		/* TRANSLATORS: the user is using Gentoo/Arch and has screwed something up */
 		return _("Required efivarfs filesystem was not found");
@@ -1719,6 +1742,7 @@ fu_util_plugin_flag_to_cli_text(FwupdPluginFlags plugin_flag)
 		return NULL;
 	case FWUPD_PLUGIN_FLAG_NONE:
 	case FWUPD_PLUGIN_FLAG_REQUIRE_HWID:
+	case FWUPD_PLUGIN_FLAG_MODULAR:
 		return fu_util_term_format(fu_util_plugin_flag_to_string(plugin_flag),
 					   FU_UTIL_TERM_COLOR_GREEN);
 	case FWUPD_PLUGIN_FLAG_DISABLED:
@@ -1729,6 +1753,7 @@ fu_util_plugin_flag_to_cli_text(FwupdPluginFlags plugin_flag)
 	case FWUPD_PLUGIN_FLAG_CAPSULES_UNSUPPORTED:
 	case FWUPD_PLUGIN_FLAG_UNLOCK_REQUIRED:
 	case FWUPD_PLUGIN_FLAG_AUTH_REQUIRED:
+	case FWUPD_PLUGIN_FLAG_SECURE_CONFIG:
 	case FWUPD_PLUGIN_FLAG_EFIVAR_NOT_MOUNTED:
 	case FWUPD_PLUGIN_FLAG_ESP_NOT_FOUND:
 	case FWUPD_PLUGIN_FLAG_KERNEL_TOO_OLD:
@@ -1855,6 +1880,58 @@ fu_util_release_flag_to_string(FwupdReleaseFlags release_flag)
 	return fwupd_release_flag_to_string(release_flag);
 }
 
+static void
+fu_util_report_add_string(FwupdReport *report, guint idt, GString *str)
+{
+	const gchar *tmp2;
+	g_autofree gchar *title = NULL;
+
+	/* TRANSLATORS: the %s is a vendor name, e.g. Lenovo */
+	title = g_strdup_printf(_("Tested by %s"), fwupd_report_get_vendor(report));
+	fu_string_append(str, idt, title, NULL);
+	if (fwupd_report_get_created(report) != 0) {
+		gint64 value = (gint64)fwupd_report_get_created(report);
+		g_autoptr(GDateTime) date = g_date_time_new_from_unix_utc(value);
+		g_autofree gchar *tmp = g_date_time_format(date, "%F");
+		/* TRANSLATORS: when the release was tested */
+		fu_string_append(str, idt + 1, _("Tested"), tmp);
+	}
+	if (fwupd_report_get_distro_id(report) != NULL) {
+		g_autoptr(GString) str2 = g_string_new(fwupd_report_get_distro_id(report));
+		if (fwupd_report_get_distro_version(report) != NULL)
+			g_string_append_printf(str2,
+					       " %s",
+					       fwupd_report_get_distro_version(report));
+		if (fwupd_report_get_distro_variant(report) != NULL)
+			g_string_append_printf(str2,
+					       " (%s)",
+					       fwupd_report_get_distro_variant(report));
+		/* TRANSLATORS: the OS the release was tested on */
+		fu_string_append(str, idt + 1, _("Distribution"), str2->str);
+	}
+	if (fwupd_report_get_version_old(report) != NULL) {
+		fu_string_append(str,
+				 idt + 1,
+				 /* TRANSLATORS: the firmware old version */
+				 _("Old version"),
+				 fwupd_report_get_version_old(report));
+	}
+	tmp2 = fwupd_report_get_metadata_item(report, "RuntimeVersion(org.freedesktop.fwupd)");
+	if (tmp2 != NULL) {
+		/* TRANSLATORS: the fwupd version the release was tested on */
+		fu_string_append(str, idt + 1, _("Version[fwupd]"), tmp2);
+	}
+}
+
+gchar *
+fu_util_report_to_string(FwupdReport *report, guint idt)
+{
+	g_autoptr(GString) str = g_string_new(NULL);
+	g_return_val_if_fail(FWUPD_IS_RELEASE(report), NULL);
+	fu_util_report_add_string(report, idt, str);
+	return g_string_free(g_steal_pointer(&str), FALSE);
+}
+
 gchar *
 fu_util_release_to_string(FwupdRelease *rel, guint idt)
 {
@@ -1862,9 +1939,10 @@ fu_util_release_to_string(FwupdRelease *rel, guint idt)
 	const gchar *tmp2;
 	GPtrArray *issues = fwupd_release_get_issues(rel);
 	GPtrArray *tags = fwupd_release_get_tags(rel);
-	GString *str = g_string_new(NULL);
+	GPtrArray *reports = fwupd_release_get_reports(rel);
 	guint64 flags = fwupd_release_get_flags(rel);
 	g_autofree gchar *desc_fb = NULL;
+	g_autoptr(GString) str = g_string_new(NULL);
 
 	g_return_val_if_fail(FWUPD_IS_RELEASE(rel), NULL);
 
@@ -1936,6 +2014,10 @@ fu_util_release_to_string(FwupdRelease *rel, guint idt)
 				 _("Urgency"),
 				 fu_util_release_urgency_to_string(tmp));
 	}
+	for (guint i = 0; i < reports->len; i++) {
+		FwupdReport *report = g_ptr_array_index(reports, i);
+		fu_util_report_add_string(report, idt + 1, str);
+	}
 	if (fwupd_release_get_details_url(rel) != NULL) {
 		fu_string_append(str,
 				 idt + 1,
@@ -2022,17 +2104,17 @@ fu_util_release_to_string(FwupdRelease *rel, guint idt)
 				 tag_strs);
 	}
 
-	return g_string_free(str, FALSE);
+	return g_string_free(g_steal_pointer(&str), FALSE);
 }
 
 gchar *
 fu_util_remote_to_string(FwupdRemote *remote, guint idt)
 {
-	GString *str = g_string_new(NULL);
 	FwupdRemoteKind kind = fwupd_remote_get_kind(remote);
 	FwupdKeyringKind keyring_kind = fwupd_remote_get_keyring_kind(remote);
 	const gchar *tmp;
 	gint priority;
+	g_autoptr(GString) str = g_string_new(NULL);
 
 	g_return_val_if_fail(FWUPD_IS_REMOTE(remote), NULL);
 
@@ -2149,7 +2231,34 @@ fu_util_remote_to_string(FwupdRemote *remote, guint idt)
 				 fwupd_remote_get_automatic_reports(remote) ? "true" : "false");
 	}
 
-	return g_string_free(str, FALSE);
+	return g_string_free(g_steal_pointer(&str), FALSE);
+}
+
+const gchar *
+fu_util_request_get_message(FwupdRequest *req)
+{
+	if (fwupd_request_has_flag(req, FWUPD_REQUEST_FLAG_ALLOW_GENERIC_MESSAGE)) {
+		if (g_strcmp0(fwupd_request_get_id(req), FWUPD_REQUEST_ID_REMOVE_REPLUG) == 0) {
+			/* TRANSLATORS: warning message shown after update has been scheduled */
+			return _("The update will continue when the device USB cable has been "
+				 "unplugged and then re-inserted.");
+		}
+		if (g_strcmp0(fwupd_request_get_id(req), FWUPD_REQUEST_ID_REMOVE_USB_CABLE) == 0) {
+			/* TRANSLATORS: warning message shown after update has been scheduled */
+			return _("The update will continue when the device USB cable has been "
+				 "unplugged.");
+		}
+		if (g_strcmp0(fwupd_request_get_id(req), FWUPD_REQUEST_ID_PRESS_UNLOCK) == 0) {
+			/* TRANSLATORS: warning message */
+			return _("Press unlock on the device to continue the update process.");
+		}
+		if (g_strcmp0(fwupd_request_get_id(req), FWUPD_REQUEST_ID_DO_NOT_POWER_OFF) == 0) {
+			/* TRANSLATORS: warning message shown after update has been scheduled */
+			return _("Do not turn off your computer or remove the AC adaptor "
+				 "while the update is in progress.");
+		}
+	}
+	return fwupd_request_get_message(req);
 }
 
 static void
@@ -2238,12 +2347,12 @@ fu_util_security_event_to_string(FwupdSecurityAttr *attr)
 		      FWUPD_SECURITY_ATTR_RESULT_TAINTED,
 		      FWUPD_SECURITY_ATTR_RESULT_NOT_TAINTED,
 		      /* TRANSLATORS: HSI event title */
-		      _("Kernel is tainted")},
+		      _("Kernel is no longer tainted")},
 		     {FWUPD_SECURITY_ATTR_ID_KERNEL_TAINTED,
 		      FWUPD_SECURITY_ATTR_RESULT_NOT_TAINTED,
 		      FWUPD_SECURITY_ATTR_RESULT_TAINTED,
 		      /* TRANSLATORS: HSI event title */
-		      _("Kernel is no longer tainted")},
+		      _("Kernel is tainted")},
 		     /* ------------------------------------------*/
 		     {FWUPD_SECURITY_ATTR_ID_KERNEL_LOCKDOWN,
 		      FWUPD_SECURITY_ATTR_RESULT_ENABLED,
